using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

namespace UWB_GUI
{
    public class UWB_Controller
    {
        public struct UWB_Buttons
        {
            public ButtonState A, B, Back, LeftShoulder, LeftStick, RightShoulder, RightStick, Start, X, Y;
        }

        public struct UWB_DPad
        {
            public ButtonState Down, Left, Right, Up;
        }

        public struct UWB_GamePadThumbSticks
        {
            public Vector2 Left;
            public Vector2 Right;

        }

        public struct UWB_GamePadTriggers
        {
            public float Left;
            public float Right;
        }

        // input support
        public struct UWB_GamePadStruct
        {
            public UWB_Buttons Buttons;
            public UWB_DPad Dpad;
            // public int PacketNumber;
            public UWB_GamePadThumbSticks ThumbSticks;
            public UWB_GamePadTriggers Triggers;
            public void SetVibration(float a, float b) { }
        }


        private UWB_GamePadStruct m_GamePad;
        private PlayerIndex m_PlayerIndex;

        public UWB_Controller(PlayerIndex playerIndex)
        {
            m_PlayerIndex = playerIndex;
            m_GamePad = new UWB_GamePadStruct();
        }

        public UWB_DPad DPad
        {
            get { return m_GamePad.Dpad; }
        }

        public UWB_GamePadThumbSticks ThumbSticks
        {
            get { return m_GamePad.ThumbSticks; }
        }

        public UWB_Buttons Buttons
        {
            get { return m_GamePad.Buttons; }
        }

        public UWB_GamePadTriggers Triggers
        {
            get { return m_GamePad.Triggers; }
        }

        public void setVibration(float a, float b)
        {
            m_GamePad.SetVibration(a, b);
        }

        public void UpdateGamePadState()
        {
            if (Microsoft.Xna.Framework.Input.GamePad.GetState(m_PlayerIndex).IsConnected)
            {
                // Buttons
                m_GamePad.Buttons.A = Microsoft.Xna.Framework.Input.GamePad.GetState(m_PlayerIndex).Buttons.A;
                m_GamePad.Buttons.B = Microsoft.Xna.Framework.Input.GamePad.GetState(m_PlayerIndex).Buttons.B;
                m_GamePad.Buttons.Back = Microsoft.Xna.Framework.Input.GamePad.GetState(m_PlayerIndex).Buttons.Back;
                m_GamePad.Buttons.LeftShoulder = Microsoft.Xna.Framework.Input.GamePad.GetState(m_PlayerIndex).Buttons.LeftShoulder;
                m_GamePad.Buttons.LeftStick = Microsoft.Xna.Framework.Input.GamePad.GetState(m_PlayerIndex).Buttons.LeftStick;
                m_GamePad.Buttons.RightShoulder = Microsoft.Xna.Framework.Input.GamePad.GetState(m_PlayerIndex).Buttons.RightShoulder;
                m_GamePad.Buttons.RightStick = Microsoft.Xna.Framework.Input.GamePad.GetState(m_PlayerIndex).Buttons.RightStick;
                m_GamePad.Buttons.Start = Microsoft.Xna.Framework.Input.GamePad.GetState(m_PlayerIndex).Buttons.Start;
                m_GamePad.Buttons.X = Microsoft.Xna.Framework.Input.GamePad.GetState(m_PlayerIndex).Buttons.X;
                m_GamePad.Buttons.Y = Microsoft.Xna.Framework.Input.GamePad.GetState(m_PlayerIndex).Buttons.Y;

                // DPad
                m_GamePad.Dpad.Up = Microsoft.Xna.Framework.Input.GamePad.GetState(m_PlayerIndex).DPad.Up;
                m_GamePad.Dpad.Down = Microsoft.Xna.Framework.Input.GamePad.GetState(m_PlayerIndex).DPad.Down;
                m_GamePad.Dpad.Left = Microsoft.Xna.Framework.Input.GamePad.GetState(m_PlayerIndex).DPad.Left;
                m_GamePad.Dpad.Right = Microsoft.Xna.Framework.Input.GamePad.GetState(m_PlayerIndex).DPad.Right;

                // ThumbSticks
                m_GamePad.ThumbSticks.Left = Microsoft.Xna.Framework.Input.GamePad.GetState(m_PlayerIndex).ThumbSticks.Left;
                m_GamePad.ThumbSticks.Right = Microsoft.Xna.Framework.Input.GamePad.GetState(m_PlayerIndex).ThumbSticks.Right;

                // Triggers
                m_GamePad.Triggers.Left = Microsoft.Xna.Framework.Input.GamePad.GetState(m_PlayerIndex).Triggers.Left;
                m_GamePad.Triggers.Right = Microsoft.Xna.Framework.Input.GamePad.GetState(m_PlayerIndex).Triggers.Right;

            }
            else
            {

                m_GamePad.Buttons.A = Keyboard.GetState().IsKeyDown(Keys.NumPad0) ? ButtonState.Pressed : ButtonState.Released;
                m_GamePad.Buttons.B = Keyboard.GetState().IsKeyDown(Keys.Decimal) ? ButtonState.Pressed : ButtonState.Released;
                m_GamePad.Buttons.Back = Keyboard.GetState().IsKeyDown(Keys.NumPad7) ? ButtonState.Pressed : ButtonState.Released;
                m_GamePad.Buttons.LeftShoulder = Keyboard.GetState().IsKeyDown(Keys.Y) ? ButtonState.Pressed : ButtonState.Released;
                m_GamePad.Buttons.LeftStick = Keyboard.GetState().IsKeyDown(Keys.N) ? ButtonState.Pressed : ButtonState.Released;
                m_GamePad.Buttons.RightShoulder = Keyboard.GetState().IsKeyDown(Keys.U) ? ButtonState.Pressed : ButtonState.Released;
                m_GamePad.Buttons.RightStick = Keyboard.GetState().IsKeyDown(Keys.M) ? ButtonState.Pressed : ButtonState.Released;
                m_GamePad.Buttons.Start = Keyboard.GetState().IsKeyDown(Keys.NumPad8) ? ButtonState.Pressed : ButtonState.Released;
                m_GamePad.Buttons.X = Keyboard.GetState().IsKeyDown(Keys.NumPad9) ? ButtonState.Pressed : ButtonState.Released;
                m_GamePad.Buttons.Y = Keyboard.GetState().IsKeyDown(Keys.NumPad6) ? ButtonState.Pressed : ButtonState.Released;

                // DPad
                m_GamePad.Dpad.Up = Keyboard.GetState().IsKeyDown(Keys.NumPad5) ? ButtonState.Pressed : ButtonState.Released;
                m_GamePad.Dpad.Down = Keyboard.GetState().IsKeyDown(Keys.NumPad1) ? ButtonState.Pressed : ButtonState.Released;
                m_GamePad.Dpad.Left = Keyboard.GetState().IsKeyDown(Keys.NumPad4) ? ButtonState.Pressed : ButtonState.Released;
                m_GamePad.Dpad.Right = Keyboard.GetState().IsKeyDown(Keys.NumPad2) ? ButtonState.Pressed : ButtonState.Released;

                // ThumbSticks
                Vector2 left = new Vector2();
                Vector2 right = new Vector2();
                left.X = Keyboard.GetState().IsKeyDown(Keys.Right) ? 1f : 0.0f;
                left.X -= Keyboard.GetState().IsKeyDown(Keys.Left) ? 1f : 0.0f;
                left.Y = Keyboard.GetState().IsKeyDown(Keys.Up) ? 1f : 0.0f;
                left.Y -= Keyboard.GetState().IsKeyDown(Keys.Down) ? 1f : 0.0f;
                m_GamePad.ThumbSticks.Left = left;

                right.X = Keyboard.GetState().IsKeyDown(Keys.K) ? 1f : 0.0f;
                right.X -= Keyboard.GetState().IsKeyDown(Keys.J) ? 1f : 0.0f;
                right.Y = Keyboard.GetState().IsKeyDown(Keys.I) ? 1f : 0.0f;
                right.Y -= Keyboard.GetState().IsKeyDown(Keys.M) ? 1f : 0.0f;
                m_GamePad.ThumbSticks.Right = right;

                // Triggers
                m_GamePad.Triggers.Left = Keyboard.GetState().IsKeyDown(Keys.RightAlt) ? 1f : 0.0f;
                m_GamePad.Triggers.Right = Keyboard.GetState().IsKeyDown(Keys.RightControl) ? 1f : 0.0f;

            }
        }
    }
}
