using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

namespace UWB_GUI
{
    public class UWB_UI_Button : UWB_UI_Element
    {
        private static readonly string CONTENT_BUTTON_PATH_NAME = "Images/Button";
        private static Texture2D BUTTON_IMAGE;
        private static readonly int BUTTON_WIDTH = 73;
        private static readonly int BUTTON_HEIGHT = 22;

        private bool m_MouseLeftClickOldState;
        private bool m_Highlighted;
        private bool m_Pressed;
        private bool m_Disabled;
        private Rectangle m_SourceImageRect;
        private Rectangle m_DrawArea;
        private Color m_TintColor;
        private UWB_UI_Container m_Container;
        private UWB_UI_Label m_Label;

        public UWB_UI_Button(string label, Vector2 upperRight, UWB_UI_GUI gui)
            : this(label, upperRight, gui.GetBaseContainer(), UWB_UI_GUI.SmallFont)
        { }

        public UWB_UI_Button(string label, Vector2 upperRight, UWB_UI_Container container, SpriteFont font)
        {
            Initialize(label, upperRight, container, font);
        }

        private void Initialize(String label, Vector2 position, UWB_UI_Container container, SpriteFont font)
        {
            m_MouseLeftClickOldState = false;
            m_Highlighted = false;
            m_Pressed = false;
            m_Disabled = false;
            m_TintColor = Color.White;
            m_Container = container;

            m_SourceImageRect = new Rectangle(0, 0, BUTTON_WIDTH, BUTTON_HEIGHT);
            m_DrawArea = new Rectangle((int)position.X + m_Container.X, (int)position.Y + m_Container.Y, BUTTON_WIDTH, BUTTON_HEIGHT);
            m_Bounds = m_DrawArea;

            m_Label = new UWB_UI_Label(label, container, font);
            m_Label.SetPosition(getFontPosition());
        }

        public static void InitializeImage(ContentManager content)
        {
            BUTTON_IMAGE = content.Load<Texture2D>(CONTENT_BUTTON_PATH_NAME);
        }

        public virtual bool IsHighlighted
        {
            get { return m_Highlighted; }
            set { m_Highlighted = value; }
        }

        public virtual Color TintColor
        {
            get { return m_TintColor; }
            set { m_TintColor = value; }
        }

        public virtual void setPosition(Vector2 position)
        {
            m_Bounds.X = (int)position.X + m_Container.X;
            m_Bounds.Y = (int)position.Y + m_Container.Y;
        }

        public virtual Vector2 getPosition()
        {
            return new Vector2(m_Bounds.X, m_Bounds.Y);
        }

        public String Text
        {
            get { return m_Label.Text; }
            set
            {
                m_Label.Text = value;
                m_Label.SetPosition(getFontPosition());
            }
        }

        private Vector2 getFontPosition()
        {
            if (m_DrawArea != Rectangle.Empty)
            {
                Vector2 textSize = m_Label.Font.MeasureString(m_Label.Text);
                Vector2 position = new Vector2(m_DrawArea.X - m_Container.X + BUTTON_WIDTH / 2 - (int)textSize.X / 2, m_DrawArea.Y - m_Container.Y + BUTTON_HEIGHT / 2 - textSize.Y / 2);
                if (m_Pressed)
                    position.Y += 1;
                return position;
            }
            return Vector2.Zero;
        }

        public override void SetHighlighted(Point mousePosition)
        {
            if (!UWB_UI_Mouse.LeftMouseButtonDown)
            {
                m_Highlighted = m_Bounds.Contains(mousePosition);
                ExecuteMouseOverListeners();
            }
        }

        public override bool SetMousePressed()
        {
            Point mousePosition = new Point((int)UWB_UI_Mouse.Position.X, (int)UWB_UI_Mouse.Position.Y);
            bool pressed = m_Bounds.Contains(mousePosition) && UWB_UI_Mouse.LeftMouseButtonDown;
            m_Pressed = pressed;
            m_MouseLeftClickOldState = pressed;
            m_Label.SetPosition(getFontPosition());
            
            return (pressed ? ExecuteMousePressedListeners() : false);
        }

        public override bool SetMouseMoved()
        {
            if (m_MouseLeftClickOldState)
            {
                Point mousePosition = new Point((int)UWB_UI_Mouse.Position.X, (int)UWB_UI_Mouse.Position.Y);
                m_Pressed = m_Bounds.Contains(mousePosition);
                m_Label.SetPosition(getFontPosition());
                if (m_Pressed)
                    return ExecuteMouseMovedListeners();
            }
            return false;
        }

        public override bool SetMouseReleased()
        {
            Point mousePosition = new Point((int)UWB_UI_Mouse.Position.X, (int)UWB_UI_Mouse.Position.Y);
            if (m_Bounds.Contains(mousePosition) && m_Pressed)
                ExecuteMouseReleasedListeners();
            else
                SetHighlighted(mousePosition);
            
            m_Pressed = false;
            m_MouseLeftClickOldState = false;
            m_Label.SetPosition(getFontPosition());

            return false;
        }

        public override void Draw(GraphicsDevice graphics, SpriteBatch spriteBatch)
        {
            if (m_IsVisible)
            {
                int horizontalOffset = 0;
                if (m_Highlighted)
                    horizontalOffset = BUTTON_WIDTH;

                if (m_Pressed)
                    horizontalOffset = BUTTON_WIDTH * 2;

                m_SourceImageRect.X = horizontalOffset;

                spriteBatch.Draw(BUTTON_IMAGE, m_DrawArea, m_SourceImageRect, m_TintColor);
                m_Label.Draw(graphics, spriteBatch);
            }
            base.ExecuteDrawListeners();
        }

        public bool Disable
        {
            get { return m_Disabled; }
            set { m_Disabled = value; }
        }

        public static int Width
        {
            get { return BUTTON_WIDTH; }
        }

        public static int Height
        {
            get { return BUTTON_HEIGHT; }
        }
    }
}
