using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

namespace UWB_GUI
{
    public class UWB_UI_CheckBox : UWB_UI_Element
    {
        private static readonly String CONTENT_CHECKBOX_PATH_NAME = "Images/CheckBox";
        private static Texture2D CHECKBOX_IMAGE;
        private static readonly int CHECKBOX_TO_LABEL_SPACING = 3;
        private static readonly int CHECKBOX_WIDTH = 13;
        private static readonly int CHECKBOX_HEIGHT = 13;

        
        protected bool m_Selected;
        protected bool m_Highlighted;
        protected Rectangle m_SourceImageRect;
        protected Rectangle m_DrawArea;
        protected Color m_TintColor;
        protected UWB_UI_Container m_Container;
        protected UWB_UI_Label m_Label;

        public UWB_UI_CheckBox(UWB_UI_Container container, SpriteFont font)
        {
            Initialize(String.Empty, Vector2.Zero, container, font);
        }

        public UWB_UI_CheckBox(String label, UWB_UI_Container container, SpriteFont font)
        {
            Initialize(label, Vector2.Zero, container, font);
        }

        public UWB_UI_CheckBox(String label, Vector2 position, UWB_UI_Container container, SpriteFont font)
        {
            Initialize(label, position, container, font);
        }

        private void Initialize(String label, Vector2 position, UWB_UI_Container container, SpriteFont font)
        {
            m_Selected = false;
            m_Highlighted = false;
            m_TintColor = Color.White;
            m_Container = container;
            
            m_SourceImageRect = new Rectangle(0, 0, CHECKBOX_WIDTH, CHECKBOX_HEIGHT);
            m_DrawArea = new Rectangle((int)position.X + m_Container.X, (int)position.Y + m_Container.Y, CHECKBOX_WIDTH, CHECKBOX_HEIGHT);
            m_Label = new UWB_UI_Label(label, new Vector2(), Color.Black, container, font);
            m_Label.SetPosition(getFontPosition());
            m_Bounds = new Rectangle((int)position.X + m_Container.X, (int)position.Y + m_Container.Y, CHECKBOX_WIDTH + CHECKBOX_TO_LABEL_SPACING + (int)font.MeasureString(label).X, getHeight());
        }

        public static void InitializeImage(ContentManager content)
        {
            CHECKBOX_IMAGE = content.Load<Texture2D>(CONTENT_CHECKBOX_PATH_NAME);
        }

        public virtual bool IsSelected
        {
            get { return m_Selected; }
            set { m_Selected = value; }
        }

        public virtual bool IsHighlighted
        {
            get { return m_Highlighted; }
            set { m_Highlighted = value; }
        }

        public virtual Color TintColor
        {
            get { return m_TintColor; }
            set { m_TintColor = value; }
        }

        public virtual Color FontColor
        {
            get { return m_Label.FontColor; }
            set { m_Label.FontColor = value; }
        }

        public virtual void setPosition(Vector2 position)
        {
            m_DrawArea.X = (int)position.X;
            m_DrawArea.Y = (int)position.Y;
            m_Label.SetPosition(getFontPosition());
            m_Bounds.X = (int)position.X + m_Container.X;
            m_Bounds.Y = (int)position.Y + m_Container.Y;
        }

        private Vector2 getFontPosition()
        {
            if (m_DrawArea != Rectangle.Empty)
            {
                return new Vector2(m_DrawArea.X - m_Container.X + CHECKBOX_WIDTH + CHECKBOX_TO_LABEL_SPACING, m_DrawArea.Y - m_Container.Y);
            }
            return Vector2.Zero;
        }

        public String Text
        {
            get { return m_Label.Text; }
            set 
            {
                m_Label.Text = value;
                m_Bounds.Width = CHECKBOX_WIDTH + CHECKBOX_TO_LABEL_SPACING + (int)m_Label.Font.MeasureString(value).X;
            }
        }

        public SpriteFont Font
        {
            get { return m_Label.Font; }
            set
            {
                m_Label.Font = value;
                m_Bounds.Height = getHeight();
            }
        }

        public virtual int getHeight()
        {
            return Math.Max(m_Label.Font.LineSpacing, m_DrawArea.Height);
        }

        public override void SetHighlighted(Point mousePosition)
        {
            m_Highlighted = m_Bounds.Contains(mousePosition);
            ExecuteMouseOverListeners();
        }

        public override void SetSelected(Point mousePosition)
        {
            if (m_Bounds.Contains(mousePosition))
            {
                m_Selected = !m_Selected;
                ExecuteMousePressedListeners();
                ExecuteCheckedChangeListeners();
            }
        }

        public override void Draw(GraphicsDevice graphics, SpriteBatch spriteBatch)
        {
            if (m_IsVisible)
            {
                int horizontalOffset = 0;
                if (m_Highlighted)
                    horizontalOffset = CHECKBOX_WIDTH;

                if (m_Selected)
                    horizontalOffset += CHECKBOX_WIDTH * 2;

                m_SourceImageRect.X = horizontalOffset;

                spriteBatch.Draw(CHECKBOX_IMAGE, m_DrawArea, m_SourceImageRect, m_TintColor);
                m_Label.Draw(graphics, spriteBatch);
            }
            ExecuteDrawListeners();
        }
    }
}
