using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
using Microsoft.Xna.Framework.Content;

namespace UWB_GUI
{
    public class UWB_UI_Container : UWB_UI_Element
    {
        private Color m_ClearColor;
        protected List<UWB_UI_Item> m_Items;
        private bool m_ClearBackground;

        public UWB_UI_Container(int x, int y, int width, int height, Color clearColor)
        {
            m_Bounds = new Rectangle(x, y, width, height);
            m_Items = new List<UWB_UI_Item>();
            m_ClearColor = clearColor;
            m_ClearBackground = true;
        }

        public void AddItem(UWB_UI_Item item)
        {
            m_Items.Add(item);
        }

        public void AddItem(String name, UWB_UI_Element item)
        {
            m_Items.Add(new UWB_UI_Item(name, item));
        }

        public UWB_UI_Element GetItem(String name)
        {
            foreach (UWB_UI_Item item in m_Items)
            {
                if (item.Name == name)
                    return item.Item;
            }
            return null;
        }

        public int X
        {
            get { return m_Bounds.X; }
            set { m_Bounds.X = value; }
        }

        public int Y
        {
            get { return m_Bounds.Y; }
            set { m_Bounds.Y = value; }
        }

        public int Width
        {
            get { return m_Bounds.Width; }
            set { m_Bounds.Width = value; }
        }

        public int Height
        {
            get { return m_Bounds.Height; }
            set { m_Bounds.Height = value; }
        }

        public bool ClearBackground
        {
            get { return m_ClearBackground; }
            set { m_ClearBackground = value; }
        }

        public virtual Color BackgroundColor
        {
            get { return m_ClearColor; }
            set { m_ClearColor = value; }
        }

        public override void SetHighlighted(Point mousePosition)
        {
            if(m_Bounds.Contains(mousePosition))
            {
                foreach (UWB_UI_Item item in m_Items)
                {
                    item.Item.SetHighlighted(mousePosition);
                }
                ExecuteMouseOverListeners();
            }
        }

        public override void SetSelected(Point mousePosition)
        {
            if (m_Bounds.Contains(mousePosition))
            {
                foreach (UWB_UI_Item item in m_Items)
                {
                    item.Item.SetSelected(mousePosition);
                }
                //ExecuteMousePressedListeners();
            }
        }

        public override void Update(GameTime gameTime)
        {
            foreach (UWB_UI_Item item in m_Items)
            {
                item.Update(gameTime);
            }
            ExecuteUpdateListeners();
        }

        public override void Draw(GraphicsDevice graphics, SpriteBatch spriteBatch)
        {
            if (m_IsVisible)
            {
                if (m_ClearBackground)
                {
                    // Create a texture with one texel that corresponds to the background color
                    uint[] texel = new uint[1];
                    texel[0] = m_ClearColor.PackedValue;
                    Texture2D texture = new Texture2D(graphics, 1, 1, 1, TextureUsage.Tiled, SurfaceFormat.Color);
                    texture.SetData<uint>(texel);

                    spriteBatch.Draw(texture, m_Bounds, Color.White);
                }

                foreach (UWB_UI_Item item in m_Items)
                {
                    //if (m_Bounds.Contains(item.Item.GetBounds()))
                    {
                        item.Draw(graphics, spriteBatch);
                    }
                }
            }
            ExecuteDrawListeners();
        }

        internal protected virtual bool LeftMouseButtonDrag(Point mousePosition)
        {
            if (m_Bounds.Contains(mousePosition))
            {
                bool consumed = false;
                UWB_UI_Item item = null;
                int i = 0;

                while (i < m_Items.Count && !consumed)
                {
                    item = m_Items[i++];
                    consumed = item.Item.SetMouseDragged();
                }

                return (consumed ? true : ExecuteMouseDraggedListeners());
            }
            return false;
        }

        internal protected virtual bool LeftMouseButtonDown(Point mousePosition)
        {
            if (m_Bounds.Contains(mousePosition))
            {
                bool consumed = false;
                UWB_UI_Item item = null;
                int i = 0;

                while (i < m_Items.Count && !consumed)
                {
                    item = m_Items[i++];
                    consumed = item.Item.SetMousePressed();
                }

                return (consumed ? true : ExecuteMousePressedListeners());
            }
            return false;
        }

        internal protected virtual bool LeftMouseButtonReleased(Point mousePosition)
        {
            if (m_Bounds.Contains(mousePosition))
            {
                bool consumed = false;
                UWB_UI_Item item = null;
                int i = 0;

                while (i < m_Items.Count && !consumed)
                {
                    item = m_Items[i++];
                    consumed = item.Item.SetMouseReleased();
                }

                return (consumed ? true : ExecuteMouseReleasedListeners());
            }
            return false;
        }

        internal protected virtual bool RightMouseButtonDrag(Point mousePosition)
        {
            if (m_Bounds.Contains(mousePosition))
            {
                bool consumed = false;
                UWB_UI_Item item = null;
                int i = 0;

                while (i < m_Items.Count && !consumed)
                {
                    item = m_Items[i++];
                    consumed = item.Item.SetMouseDragged();
                }

                return (consumed ? true : ExecuteMouseDraggedListeners());
            }
            return false;
        }

        internal protected virtual bool RightMouseButtonDown(Point mousePosition)
        {
            if (m_Bounds.Contains(mousePosition))
            {
                bool consumed = false;
                UWB_UI_Item item = null;
                int i = 0;

                while (i < m_Items.Count && !consumed)
                {
                    item = m_Items[i++];
                    consumed = item.Item.SetMousePressed();
                }

                return (consumed ? true : ExecuteMousePressedListeners());
            }
            return false;
        }

        internal protected virtual bool RightMouseButtonReleased(Point mousePosition)
        {
            if (m_Bounds.Contains(mousePosition))
            {
                bool consumed = false;
                UWB_UI_Item item = null;
                int i = 0;

                while (i < m_Items.Count && !consumed)
                {
                    item = m_Items[i++];
                    consumed = item.Item.SetMouseReleased();
                }

                return (consumed ? true : ExecuteMouseReleasedListeners());
            }
            return false;
        }

        internal protected virtual bool MouseMoved(Point mousePosition)
        {
            if (m_Bounds.Contains(mousePosition))
            {
                bool consumed = false;
                UWB_UI_Item item = null;
                int i = 0;

                while (i < m_Items.Count && !consumed)
                {
                    item = m_Items[i++];
                    consumed = item.Item.SetMouseMoved();
                }

                return (consumed ? true : ExecuteMouseMovedListeners());
            }
            return false;
        }

        internal protected virtual bool MiddleMouseButtonDown(Point mousePosition)
        {
            if (m_Bounds.Contains(mousePosition))
            {
                bool consumed = false;
                UWB_UI_Item item = null;
                int i = 0;

                while (i < m_Items.Count && !consumed)
                {
                    item = m_Items[i++];
                    consumed = item.Item.SetMousePressed();
                }

                return (consumed ? true : ExecuteMousePressedListeners());
            }
            return false;
        }

        internal protected virtual bool MiddleMouseButtonReleased(Point mousePosition)
        {
            if (m_Bounds.Contains(mousePosition))
            {
                bool consumed = false;
                UWB_UI_Item item = null;
                int i = 0;

                while (i < m_Items.Count && !consumed)
                {
                    item = m_Items[i++];
                    consumed = item.Item.SetMouseReleased();
                }

                return (consumed ? true : ExecuteMouseReleasedListeners());
            }
            return false;
        }

        internal protected virtual bool MiddleMouseButtonDrag(Point mousePosition)
        {
            if (m_Bounds.Contains(mousePosition))
            {
                bool consumed = false;
                UWB_UI_Item item = null;
                int i = 0;

                while (i < m_Items.Count && !consumed)
                {
                    item = m_Items[i++];
                    consumed = item.Item.SetMouseDragged();
                }

                return (consumed ? true : ExecuteMouseDraggedListeners());
            }
            return false;
        }

    }
}
