using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace UWB_GUI
{
    public abstract class UWB_UI_Element
    {
        protected Rectangle m_Bounds = new Rectangle();
        protected bool m_IsVisible = true;
        protected List<UWB_UI_Listener> m_Listeners = new List<UWB_UI_Listener>();

        public virtual Rectangle GetBounds()
        {
            return m_Bounds;
        }

        public virtual bool IsVisible
        {
            set { m_IsVisible = value; }
            get { return m_IsVisible; }
        }

        public virtual List<UWB_UI_Listener> GetListeners()
        {
            return m_Listeners;
        }

        public virtual void ClearListener()
        {
            m_Listeners.Clear();
        }

        public virtual void AddListener(UWB_UI_Listener listener)
        {
            m_Listeners.Add(listener);
        }

        public virtual void RemoveListener(UWB_UI_Listener listener)
        {
            m_Listeners.Remove(listener);
        }

        public virtual void SetSelected(Point mousePosition) { }
        public virtual void SetHighlighted(Point mousePosition) { }
        public virtual bool SetMousePressed() { return ExecuteMousePressedListeners(); }
        public virtual bool SetMouseDragged() { return ExecuteMouseDraggedListeners(); }
        public virtual bool SetMouseReleased() { return ExecuteMouseReleasedListeners(); }
        public virtual bool SetMouseMoved() { return ExecuteMouseMovedListeners(); }

        public virtual void Update(GameTime gameTime) { ExecuteUpdateListeners(); }
        public abstract void Draw(GraphicsDevice graphics, SpriteBatch spriteBatch);

        protected virtual void ExecuteMouseOverListeners()
        {
            foreach (UWB_UI_Listener listener in m_Listeners)
            {
                listener.MouseOver(this);
            }
        }

        protected virtual bool ExecuteMousePressedListeners()
        {
            bool consumed = false;
            UWB_UI_Listener listener = null;
            int i = 0;

            while (i < m_Listeners.Count && !consumed)
            {
                listener = m_Listeners[i++];
                consumed = listener.MousePressed(this);
            }

            return consumed;
        }

        protected virtual bool ExecuteValueChangedListeners()
        {
            bool consumed = false;
            UWB_UI_Listener listener = null;
            int i = 0;

            while (i < m_Listeners.Count && !consumed)
            {
                listener = m_Listeners[i++];
                consumed = listener.ValueChanged(this);
            }

            return consumed;
        }

        protected virtual void ExecuteCheckedChangeListeners()
        {
            foreach (UWB_UI_Listener listener in m_Listeners)
            {
                listener.CheckedChange(this);
            }
        }

        protected virtual void ExecuteUpdateListeners()
        {
            foreach (UWB_UI_Listener listener in m_Listeners)
            {
                listener.Update(this);
            }
        }

        protected virtual void ExecuteDrawListeners()
        {
            foreach (UWB_UI_Listener listener in m_Listeners)
            {
                listener.Draw(this);
            }
        }

        protected virtual bool ExecuteMouseDraggedListeners()
        {
            bool consumed = false;
            UWB_UI_Listener listener = null;
            int i = 0;

            while (i < m_Listeners.Count && !consumed)
            {
                listener = m_Listeners[i++];
                consumed = listener.MouseDragged(this);
            }

            return consumed;
        }

        protected virtual bool ExecuteMouseReleasedListeners()
        {
            bool consumed = false;
            UWB_UI_Listener listener = null;
            int i = 0;

            while (i < m_Listeners.Count && !consumed)
            {
                listener = m_Listeners[i++];
                consumed = listener.MouseReleased(this);
            }

            return consumed;
        }

        protected virtual bool ExecuteMouseMovedListeners()
        {
            bool consumed = false;
            UWB_UI_Listener listener = null;
            int i = 0;

            while (i < m_Listeners.Count && !consumed)
            {
                listener = m_Listeners[i++];
                consumed = listener.MouseMoved(this);
            }

            return consumed;
        }
    }
}
