using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

namespace UWB_GUI
{
    public class UWB_UI_GUI
    {
        private static GraphicsDevice m_Graphics;
        private SpriteBatch m_SpriteBatch;
        private ResolveTexture2D m_ResolveTexture;
        protected UWB_UI_Container m_BaseContainer;
        private UWB_UI_Mouse m_Mouse;

        private UWB_Controller m_Controller;
        private bool m_MouseLeftClickOldState;
        private bool m_MouseRightClickOldState;
        private bool m_MouseMiddleClickOldState;
        private bool m_StartOldState;

        private static SpriteFont m_smFont;
        private static SpriteFont m_lgFont;
        private static SpriteFont m_titleFont;

        public UWB_UI_GUI(Game game)
        {
            m_Graphics = game.GraphicsDevice;
            ContentManager content = game.Content;
            //content.RootDirectory = "Content";

            UWB_UI_Label.InitializeImage(content);
            UWB_UI_CheckBox.InitializeImage(content);
            UWB_UI_RadioButton.InitializeImage(content);
            UWB_UI_TreeViewNode.InitializeImage(content);
            UWB_UI_SliderBar.InitializeImage(content);
            UWB_UI_Button.InitializeImage(content);

            //Load Font
            m_smFont = content.Load<SpriteFont>("Font/Arial-8");
            m_lgFont = content.Load<SpriteFont>("Font/Arial-14");
            m_titleFont = content.Load<SpriteFont>("Font/ComicSans-14");

            m_Controller = new UWB_Controller(PlayerIndex.One);

            m_SpriteBatch = new SpriteBatch(m_Graphics);
            m_BaseContainer = new UWB_UI_Container(m_Graphics.Viewport.X, m_Graphics.Viewport.Y, m_Graphics.Viewport.Width, m_Graphics.Viewport.Height, Color.TransparentWhite);
            m_BaseContainer.ClearBackground = false;
            m_Mouse = new UWB_UI_Mouse();
            UWB_UI_Mouse.Init(m_BaseContainer, content);

            m_ResolveTexture = new ResolveTexture2D(m_Graphics, m_Graphics.Viewport.Width, m_Graphics.Viewport.Height, 1, SurfaceFormat.Color);
            m_MouseLeftClickOldState = false;
            m_MouseRightClickOldState = false;
            m_MouseMiddleClickOldState = false;
        }

        public bool IsVisible
        {
            get { return m_BaseContainer.IsVisible; }
            set { m_BaseContainer.IsVisible = value; }
        }

        public UWB_UI_Container GetBaseContainer()
        {
            return m_BaseContainer;
        }

        public void AddItem(UWB_UI_Item item)
        {
            m_BaseContainer.AddItem(item);
        }

        public void AddItem(String name, UWB_UI_Element item)
        {
            m_BaseContainer.AddItem(new UWB_UI_Item(name, item));
        }

        public UWB_UI_Element GetItem(String name)
        {
            return m_BaseContainer.GetItem(name);
        }

        public void Update(GameTime gameTime)
        {
            m_Controller.UpdateGamePadState();

            KeyboardState newKeyBoardState = Keyboard.GetState();

            if (!m_BaseContainer.IsVisible)
            {
            }
            else
            {
                UWB_UI_Mouse.LeftMouseButtonDown = m_Controller.Buttons.A == ButtonState.Pressed;
                UWB_UI_Mouse.RightMouseButtonDown = m_Controller.Buttons.B == ButtonState.Pressed;
                UWB_UI_Mouse.MiddleMouseButtonDown = m_Controller.Buttons.Y == ButtonState.Pressed;

                UWB_UI_Mouse.MoveMousePosition(m_Controller.ThumbSticks.Left * new Vector2(1, -1));

                Point mousePos = new Point((int)UWB_UI_Mouse.Position.X, (int)UWB_UI_Mouse.Position.Y);

                if (UWB_UI_Mouse.HasMouseMoved)
                    MouseMoved(mousePos);

                if (UWB_UI_Mouse.LeftMouseButtonDown)
                {
                    if (!m_MouseLeftClickOldState)
                        LeftMouseButtonDown(mousePos);
                    else if (UWB_UI_Mouse.HasMouseMoved)
                        LeftMouseButtonDrag(mousePos);
                }
                else if (m_MouseLeftClickOldState)
                    LeftMouseButtonReleased(mousePos);

                if (UWB_UI_Mouse.RightMouseButtonDown)
                {
                    if (!m_MouseRightClickOldState)
                        RightMouseButtonDown(mousePos);
                    else if (UWB_UI_Mouse.HasMouseMoved)
                        RightMouseButtonDrag(mousePos);
                }
                else if (m_MouseRightClickOldState)
                    RightMouseButtonReleased(mousePos);

                if (UWB_UI_Mouse.MiddleMouseButtonDown)
                {
                    if (!m_MouseMiddleClickOldState)
                        MiddleMouseButtonDown(mousePos);
                    else if (UWB_UI_Mouse.HasMouseMoved)
                        MiddleMouseButtonDrag(mousePos);
                }
                else if (m_MouseMiddleClickOldState)
                    MiddleMouseButtonReleased(mousePos);

                m_BaseContainer.Update(gameTime);
            }

            if (m_Controller.Buttons.Start == ButtonState.Pressed)
            {

            }
            else if (m_StartOldState)
            {
                m_BaseContainer.IsVisible = !m_BaseContainer.IsVisible;
            }

            UWB_UI_Mouse.HasMouseMoved = false;
            m_MouseLeftClickOldState = UWB_UI_Mouse.LeftMouseButtonDown;
            m_MouseRightClickOldState = UWB_UI_Mouse.RightMouseButtonDown;
            m_MouseMiddleClickOldState = UWB_UI_Mouse.MiddleMouseButtonDown;
            m_StartOldState = m_Controller.Buttons.Start == ButtonState.Pressed;
        }

        protected virtual bool MiddleMouseButtonDown(Point mousePos)
        {
            return m_BaseContainer.MiddleMouseButtonDown(mousePos);
        }

        protected virtual bool MiddleMouseButtonReleased(Point mousePos)
        {
            m_BaseContainer.SetSelected(mousePos);
            return m_BaseContainer.MiddleMouseButtonReleased(mousePos);
        }

        protected virtual bool MiddleMouseButtonDrag(Point mousePos)
        {
            return m_BaseContainer.MiddleMouseButtonDrag(mousePos);
        }

        protected virtual bool LeftMouseButtonReleased(Point mousePos)
        {
            m_BaseContainer.SetSelected(mousePos);
            return m_BaseContainer.LeftMouseButtonReleased(mousePos);
        }

        protected virtual bool LeftMouseButtonDrag(Point mousePos)
        {
            return m_BaseContainer.LeftMouseButtonDrag(mousePos);
        }

        protected virtual bool LeftMouseButtonDown(Point mousePos)
        {
            return m_BaseContainer.LeftMouseButtonDown(mousePos);
        }

        protected virtual bool RightMouseButtonReleased(Point mousePos)
        {
            m_BaseContainer.SetSelected(mousePos);
            return m_BaseContainer.RightMouseButtonReleased(mousePos);
        }

        protected virtual bool RightMouseButtonDrag(Point mousePos)
        {
            return m_BaseContainer.RightMouseButtonDrag(mousePos);
        }

        protected virtual bool RightMouseButtonDown(Point mousePos)
        {
            return m_BaseContainer.RightMouseButtonDown(mousePos);
        }

        protected virtual bool MouseMoved(Point mousePos)
        {
            m_BaseContainer.SetHighlighted(mousePos);
            return m_BaseContainer.MouseMoved(mousePos);
        }

        public void Draw()
        {
            m_Graphics.ResolveBackBuffer(m_ResolveTexture);

            m_SpriteBatch.Begin(SpriteBlendMode.AlphaBlend, SpriteSortMode.Deferred, SaveStateMode.SaveState);
            m_SpriteBatch.Draw(m_ResolveTexture, Vector2.Zero, Color.White);
            
            if (m_BaseContainer.IsVisible)
            {
                m_BaseContainer.Draw(m_Graphics, m_SpriteBatch);
                m_Mouse.Draw(null, m_SpriteBatch);
            }
            m_SpriteBatch.End();
        }

        public UWB_UI_Mouse Mouse
        {
            get { return m_Mouse; }
        }

        public static SpriteFont SmallFont
        {
            get { return m_smFont; }
        }

        public static SpriteFont LargeFont
        {
            get { return m_lgFont; }
        }

        internal static GraphicsDevice Graphics
        {
            get { return m_Graphics; }
        }
    }
}
