using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

namespace UWB_GUI
{
    public class UWB_UI_Label : UWB_UI_Element
    {
        private static readonly String CONTENT_CHECKBOX_PATH_NAME = "Images/LabelSelectedBackground";
        private static Texture2D SELECTED_BACKGROUND_IMAGE;
        private static Rectangle SOURCE_BACKGROUND_IMAGE_RECTANGLE = new Rectangle(0, 0, 1, 1);
        private static int BACKGROUND_BORDER_LENGTH = 2;

        private String m_Text;
        private SpriteFont m_SpriteFont;
        private Vector2 m_Position;
        private Color m_FontColor;
        private UWB_UI_Container m_Container;
        private Color m_SelectedColor;
        private Rectangle m_BackgroundDrawArea;

        private bool m_DrawSelectedBackground;

        public UWB_UI_Label(String text, UWB_UI_Container container, SpriteFont spriteFont)
        {
            Initialize(container, text, new Vector2(), Color.Black, spriteFont);
        }

        public UWB_UI_Label(String text, Vector2 position, UWB_UI_Container container, SpriteFont spriteFont)
        {
            Initialize(container, text, position, Color.Black, spriteFont);
        }

        public UWB_UI_Label(String text, Vector2 position, Color color, UWB_UI_Container container, SpriteFont spriteFont)
        {
            Initialize(container, text, position, color, spriteFont);
        }

        private void Initialize(UWB_UI_Container container, String text, Vector2 position, Color color, SpriteFont spriteFont)
        {
            m_Container = container;
            m_Text = text;
            m_Position = position + new Vector2(m_Container.X, m_Container.Y);
            m_FontColor = color;
            m_SpriteFont = spriteFont;
            Vector2 textDimensions = m_SpriteFont.MeasureString(text);
            m_Bounds = new Rectangle((int)m_Position.X, (int)m_Position.Y, (int)textDimensions.X, (int)textDimensions.Y);
            m_BackgroundDrawArea = new Rectangle((int)m_Position.X - BACKGROUND_BORDER_LENGTH, (int)m_Position.Y, (int)textDimensions.X + BACKGROUND_BORDER_LENGTH * 2, (int)textDimensions.Y);
            m_DrawSelectedBackground = false;
            m_SelectedColor = Color.LightBlue;

        }

        public static void InitializeImage(ContentManager content)
        {
            SELECTED_BACKGROUND_IMAGE = content.Load<Texture2D>(CONTENT_CHECKBOX_PATH_NAME);
        }

        public Vector2 Position
        {
            get { return m_Position; }
        }

        public Boolean DrawSelectedBackground
        {
            get { return m_DrawSelectedBackground; }
            set { m_DrawSelectedBackground = value; }
        }

        public void SetPosition(float posX, float posY)
        {
            m_Position.X = posX + m_Container.X;
            m_Position.Y = posY + m_Container.Y;
            m_Bounds.X = (int)m_Position.X;
            m_Bounds.Y = (int)m_Position.Y;
            m_BackgroundDrawArea.X = m_Bounds.X - BACKGROUND_BORDER_LENGTH;
            m_BackgroundDrawArea.Y = m_Bounds.Y;
        }

        public void SetPosition(Vector2 pos)
        {
            SetPosition(pos.X, pos.Y);
        }

        public String Text
        {
            get { return m_Text; }
            set
            {
                m_Text = value;
                m_Bounds.Width = (int)m_SpriteFont.MeasureString(m_Text).X;
                m_BackgroundDrawArea.Width = m_Bounds.Width + BACKGROUND_BORDER_LENGTH * 2;
            }
        }

        public Color FontColor
        {
            get { return m_FontColor; }
            set { m_FontColor = value; }
        }

        public SpriteFont Font
        {
            get { return m_SpriteFont; }
            set
            {
                m_SpriteFont = value;
                m_Bounds.Height = m_SpriteFont.LineSpacing;
                m_BackgroundDrawArea.Height = m_SpriteFont.LineSpacing;
            }
        }

        public Color SelectedColor
        {
            get { return m_SelectedColor; }
            set { m_SelectedColor = value; }
        }

        public override void Draw(GraphicsDevice graphics, SpriteBatch spriteBatch)
        {
            if (m_IsVisible)
            {


                if (m_DrawSelectedBackground)
                    spriteBatch.Draw(SELECTED_BACKGROUND_IMAGE, m_BackgroundDrawArea, m_SelectedColor);

                spriteBatch.DrawString(m_SpriteFont, m_Text, m_Position, m_FontColor);
            }
        }
    }
}
