using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
using Microsoft.Xna.Framework.Content;

namespace UWB_GUI
{
    public class UWB_UI_Mouse : UWB_UI_Element
    {
        private static readonly String CONTENT_MOUSE_PATH_NAME = "Images/MousePointer";
        
        private static Texture2D m_MouseImage;
        private static Vector2 m_Position;
        private static UWB_UI_Container m_ParentContainer;
        private static Rectangle m_DrawArea;
        private static Color m_Color;
        private static float m_Sensitivity;
        private static bool m_HasMouseMoved;
        private static bool m_LeftMouseButtonDown;
        private static bool m_RightMouseButtonDown;
        private static bool m_MiddleMouseButtonDown;
        private new static Rectangle m_Bounds;

        public static void Init(UWB_UI_Container parentContainer, ContentManager content)
        {
            m_MouseImage = content.Load<Texture2D>(CONTENT_MOUSE_PATH_NAME);
            m_Position = new Vector2(parentContainer.X + parentContainer.Width/2.0f, 
                                    parentContainer.Y + parentContainer.Height/2.0f);
            m_Color = Color.White;
#if XBOX360
            m_Sensitivity = 5f;
#else
            m_Sensitivity = 1f;
#endif
            m_ParentContainer = parentContainer;
            m_DrawArea = new Rectangle(0, 0, m_MouseImage.Width, m_MouseImage.Height);
            m_Bounds = new Rectangle(0, 0, m_MouseImage.Width, m_MouseImage.Height);
            m_HasMouseMoved = false;
            m_LeftMouseButtonDown = false;
            m_RightMouseButtonDown = false;
        }

        public static void MoveMousePosition(Vector2 delta)
        {
            int temp = m_DrawArea.Left;
            float tempX = m_Position.X + (delta.X * m_Sensitivity);
            if (tempX <= (m_ParentContainer.Width + m_ParentContainer.X) &&
                tempX >= m_ParentContainer.X &&
                Math.Abs(tempX - m_Position.X) > 0.5f)
            {
                m_Position += new Vector2(delta.X * m_Sensitivity, 0);
                m_HasMouseMoved = true;
            }

            float tempY = m_Position.Y + (delta.Y * m_Sensitivity);
            if (tempY <= (m_ParentContainer.Height + m_ParentContainer.Y) &&
                tempY >= m_ParentContainer.Y &&
                Math.Abs(tempY - m_Position.Y) > 0.5f)
            {
                m_Position += new Vector2(0, delta.Y * m_Sensitivity);
                m_HasMouseMoved = true;
            }
        }

        public static bool HasMouseMoved
        {
            get { return m_HasMouseMoved; }
            set { m_HasMouseMoved = value; }
        }

        public static Vector2 Position
        {
            get { return m_Position; }
            set { m_Position = value; }
        }

        public static bool LeftMouseButtonDown
        {
            get { return m_LeftMouseButtonDown; }
            set { m_LeftMouseButtonDown = value; }
        }

        public static bool RightMouseButtonDown
        {
            get { return m_RightMouseButtonDown; }
            set { m_RightMouseButtonDown = value; }
        }

        public static bool MiddleMouseButtonDown
        {
            get { return m_MiddleMouseButtonDown; }
            set { m_MiddleMouseButtonDown = value; }
        }

        public static Color TintColor
        {
            get { return m_Color; }
            set { m_Color = value; }
        }

        public static float Sensitivity
        {
            get { return m_Sensitivity; }
            set { m_Sensitivity = value; }
        }

        public new static Rectangle GetBounds()
        {
            m_Bounds.X = (int)m_Position.X;
            m_Bounds.Y = (int)m_Position.Y;
            return m_Bounds;
        }

        public override void Update(GameTime gameTime)
        {
            base.Update(gameTime);
        }

        public override void Draw(GraphicsDevice graphics, SpriteBatch spriteBatch)
        {
            if (m_IsVisible)
            {
                m_DrawArea.X = (int)m_Position.X;
                m_DrawArea.Y = (int)m_Position.Y;
                spriteBatch.Draw(m_MouseImage, m_DrawArea, m_Color);
            }
        }
    }
}
