using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

namespace UWB_GUI
{
    public class UWB_UI_RadioButton : UWB_UI_CheckBox
    {
        private static readonly String CONTENT_RADIOBUTTON_PATH_NAME = "Images/RadioButton";
        private static Texture2D RADIOBOX_IMAGE;
        private static readonly int RADIOBUTTON_TO_LABEL_SPACING = 3;
        private static readonly int RADIOBUTTON_WIDTH = 12;
        private static readonly int RADIOBUTTON_HEIGHT = 12;

        public UWB_UI_RadioButton(UWB_UI_Container container, SpriteFont font)
            : base(container, font)
        {
            Initialize(String.Empty, Vector2.Zero, container, font);
        }

        public UWB_UI_RadioButton(String label, UWB_UI_Container container, SpriteFont font) 
            : base(label, container, font)
        {
            Initialize(label, Vector2.Zero, container, font);
        }

        public UWB_UI_RadioButton(String label, Vector2 position, UWB_UI_Container container, SpriteFont font)
            : base(label, position, container, font)
        {
            Initialize(label, position, container, font);
        }

        private void Initialize(String label, Vector2 position, UWB_UI_Container container, SpriteFont font)
        {
            m_DrawArea = new Rectangle((int)position.X + m_Container.X, (int)position.Y + m_Container.Y, RADIOBUTTON_WIDTH, RADIOBUTTON_HEIGHT);
            m_Bounds = new Rectangle((int)position.X + m_Container.X, (int)position.Y + m_Container.Y, RADIOBUTTON_WIDTH + RADIOBUTTON_TO_LABEL_SPACING + (int)font.MeasureString(label).X, getHeight());
            m_SourceImageRect = new Rectangle(0, 0, RADIOBUTTON_WIDTH, RADIOBUTTON_HEIGHT);
        }

        public new static void InitializeImage(ContentManager content)
        {
            RADIOBOX_IMAGE = content.Load<Texture2D>(CONTENT_RADIOBUTTON_PATH_NAME);
        }

        public override void SetHighlighted(Point mousePosition)
        {
            base.SetHighlighted(mousePosition);
            ExecuteMouseOverListeners();
        }

        public override void Draw(GraphicsDevice graphics, SpriteBatch spriteBatch)
        {
            if (m_IsVisible)
            {
                int horizontalOffset = 0;
                if (m_Highlighted)
                    horizontalOffset = RADIOBUTTON_WIDTH;

                if (m_Selected)
                    horizontalOffset += RADIOBUTTON_WIDTH * 2;

                m_SourceImageRect.X = horizontalOffset;

                spriteBatch.Draw(RADIOBOX_IMAGE, m_DrawArea, m_SourceImageRect, m_TintColor);
                m_Label.Draw(graphics, spriteBatch);
            }
            ExecuteDrawListeners();
        }
    }
}
