using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

namespace UWB_GUI
{
    public class UWB_UI_SliderBar : UWB_UI_Element
    {
        private static readonly String CONTENT_SLIDERBAR_SELECTOR_PATH_NAME = "Images/SliderBarSelector";
        private static readonly String CONTENT_SLIDERBAR_PATH_NAME = "Images/SliderBar";
        private static Texture2D SELECTOR_IMAGE;
        private static Texture2D BAR_IMAGE;
        private static readonly int SLIDERBAR_SELECTOR_WIDTH = 10;
        private static readonly int SLIDERBAR_SELECTOR_HEIGHT = 18;
        private static readonly int SLIDERBAR_WIDTH = 88;
        private static readonly int SLIDERBAR_HEIGHT = 4;

        private bool m_MouseLeftClickOldState;
        private bool m_SelectorFreeMovement;
        private bool m_Highlighted;
        private bool m_Disabled;
        private Rectangle m_SelectorImageRect;
        private Rectangle m_BarImageRect;
        private Rectangle m_SelectorDrawArea;
        private Rectangle m_BarDrawArea;
        private Color m_TintColor;
        private UWB_UI_Container m_Container;
        private Vector2 m_Position;
        private int m_Width;
        private int m_Value;
        private int m_MinValue;
        private int m_MaxValue;
        private int m_MinChange;
        private float m_LargeChange;
        private int m_PrevValue;
        private int m_PrevDelta;
        private int m_RangeThreshold;

        public UWB_UI_SliderBar(UWB_UI_Container container)
        {
            Initialize(Vector2.Zero, 100, container);
        }

        public UWB_UI_SliderBar(Vector2 position, int width, UWB_UI_Container container)
        {
            Initialize(position, width, container);
        }

        private void Initialize(Vector2 position, int width, UWB_UI_Container container)
        {
            m_MouseLeftClickOldState = false;
            m_SelectorFreeMovement = false;
            m_Highlighted = false;
            m_Disabled = false;
            m_TintColor = Color.White;
            m_Container = container;
            m_MinValue = 0;
            m_MaxValue = 100;
            m_Width = width;
            m_Value = 50;
            m_MinChange = 1;
            m_LargeChange = 0.01f;
            m_Position = position;
            m_SelectorImageRect = new Rectangle(0, 0, SLIDERBAR_SELECTOR_WIDTH, SLIDERBAR_SELECTOR_HEIGHT);
            m_BarImageRect = new Rectangle(0, 0, SLIDERBAR_WIDTH, SLIDERBAR_HEIGHT);
            m_SelectorDrawArea = new Rectangle((int)position.X + m_Container.X, (int)position.Y + m_Container.Y, SLIDERBAR_SELECTOR_WIDTH, SLIDERBAR_SELECTOR_HEIGHT);
            m_BarDrawArea = new Rectangle((int)position.X + m_Container.X, (int)position.Y + m_Container.Y + (SLIDERBAR_SELECTOR_HEIGHT / 3), SLIDERBAR_WIDTH, SLIDERBAR_HEIGHT);
            m_Bounds = new Rectangle((int)position.X + m_Container.X, (int)position.Y + m_Container.Y, width + 1, m_SelectorDrawArea.Height);
            m_PrevValue = m_Value;
            m_RangeThreshold = 10;
            m_PrevDelta = 0;
        }

        public static void InitializeImage(ContentManager content)
        {
            SELECTOR_IMAGE = content.Load<Texture2D>(CONTENT_SLIDERBAR_SELECTOR_PATH_NAME);
            BAR_IMAGE = content.Load<Texture2D>(CONTENT_SLIDERBAR_PATH_NAME);
        }

        public void SetValue(int val)
        {
            if (val >= m_MinValue && val <= m_MaxValue)
            {
                m_Value = val;
                m_PrevValue = val;
                ExecuteValueChangedListeners();
            }
        }

        public int Value
        {
            get { return m_Value; }
            set
            {
                m_Value = value;
                m_PrevValue = value;
            }
        }

        public void SetMaxValue(int val)
        {
            if (val >= m_MinValue)
            {
                m_MaxValue = val;
                m_Disabled = (val == m_MinValue) ? true : false;
            }
        }

        public int MaxValue
        {
            get { return m_MaxValue; }
        }

        public void SetMinValue(int val)
        {
            if (val <= m_MaxValue)
            {
                m_MinValue = val;
                m_Disabled = (val == m_MaxValue) ? true : false;
            }
        }

        public int MinValue
        {
            get { return m_MinValue; }
        }

        public void SetWidth(int val)
        {
            if (val > 0)
            {
                m_Width = val;
                m_Bounds.Width = val + 1;
            }
        }

        public int MinIncrement
        {
            get { return m_MinChange; }
            set { m_MinChange = value; }
        }

        public int Width
        {
            get { return m_Width; }
        }

        public virtual bool IsHighlighted
        {
            get { return m_Highlighted; }
            set { m_Highlighted = value; }
        }

        public virtual Color TintColor
        {
            get { return m_TintColor; }
            set { m_TintColor = value; }
        }

        public virtual void setPosition(Vector2 position)
        {
            m_SelectorDrawArea.X = (int)position.X + m_Container.X;
            m_SelectorDrawArea.Y = (int)position.Y + m_Container.Y;
            m_BarDrawArea.X = (int)position.X + m_Container.X;
            m_BarDrawArea.Y = (int)position.Y + m_Container.Y;
            m_Bounds.X = (int)position.X + m_Container.X;
            m_Bounds.Y = (int)position.Y + m_Container.Y;
            m_Position = position;
        }

        public virtual Vector2 getPosition()
        {
            return m_Position;
        }

        public virtual int getHeight()
        {
            return m_SelectorDrawArea.Height;
            //return Math.Max(m_SpriteFont.LineSpacing, m_BarDrawArea.Height);
        }

        public override void SetHighlighted(Point mousePosition)
        {
            m_Highlighted = m_Bounds.Contains(mousePosition);
            ExecuteMouseOverListeners();
        }

        public override void Update(GameTime gameTime)
        {
            if (UWB_UI_Mouse.LeftMouseButtonDown)
            {
                Vector2 mousePosition = UWB_UI_Mouse.Position;
                bool inBounds = m_Bounds.Contains((int)mousePosition.X, (int)mousePosition.Y);
                if (!inBounds)
                {
                    m_MouseLeftClickOldState = false;
                    m_SelectorFreeMovement = false;
                    return;
                }

                if (!m_Disabled && mousePosition.X >= m_BarDrawArea.X && mousePosition.X <= (m_BarDrawArea.X + m_BarDrawArea.Width))
                {
                    int selectorDelta = (int)(mousePosition.X - m_SelectorDrawArea.X - m_SelectorDrawArea.Width / 2f);

                    // if selector is freely moving, then update value based on selector position
                    if (m_SelectorFreeMovement)
                    {
                        int selectorValue = Convert.ToInt32(((mousePosition.X - m_BarDrawArea.X - m_PrevDelta) / m_BarDrawArea.Width) * (m_MaxValue - m_MinValue)) + m_MinValue;
                        if (selectorValue != m_Value)
                        {
                            if (selectorValue > m_MaxValue)
                                selectorValue = m_MaxValue;
                            else if (selectorValue < m_MinValue)
                                selectorValue = m_MinValue;

                            m_PrevValue = m_Value;
                            m_Value = selectorValue;
                            ExecuteValueChangedListeners();
                        }
                    }
                    else
                    // otherwise, update value based on mouse position
                    {
                        int valueRange = m_MaxValue - m_MinValue;
                        float newValue = (mousePosition.X - m_BarDrawArea.X) / m_BarDrawArea.Width * valueRange + m_MinValue;
                        float valueDelta = Math.Abs(newValue - m_Value);
                        float largeIncrement = m_LargeChange * valueRange;
                        float curIncrement = m_MinChange;

                        if (newValue < m_Value)
                            curIncrement = -curIncrement;

                        if ((Math.Abs(selectorDelta) > (m_SelectorDrawArea.Width / 2))  // if mouse pos is not on the selector
                            && (!m_MouseLeftClickOldState                               // if mouse button was already down
                            || ((m_Value + curIncrement != m_PrevValue)                 // if new value is not the previous value (to prevent values from alternating every update)
                            && (valueRange > m_RangeThreshold))))                       // if range is small (to prevent slider from changing values every update)
                        {
                            m_PrevValue = m_Value;

                            if (valueDelta >= largeIncrement)
                                m_Value += (int)curIncrement;
                            else
                                m_Value = (int)newValue;

                            ExecuteValueChangedListeners();
                        }
                    }
                }
                ExecuteMousePressedListeners();
                m_MouseLeftClickOldState = true;
            }
            else
            {
                m_MouseLeftClickOldState = false;
                m_SelectorFreeMovement = false;
            }

            //ExecuteUpdateListeners();
        }

        public override void Draw(GraphicsDevice graphics, SpriteBatch spriteBatch)
        {
            if (m_IsVisible)
            {
                int horizontalOffset = 0;
                if (m_Highlighted)
                    horizontalOffset = SLIDERBAR_SELECTOR_WIDTH;

                m_SelectorImageRect.X = horizontalOffset;

                m_BarDrawArea.Width = m_Width;

                Vector2 mousePosition = UWB_UI_Mouse.Position;
                int selectorDelta = m_SelectorFreeMovement ? m_PrevDelta : (int)(mousePosition.X - m_SelectorDrawArea.X - m_SelectorDrawArea.Width / 2f);

                // if mouse is clicked on selector, move selector to follow mouse
                if (UWB_UI_Mouse.LeftMouseButtonDown && m_Bounds.Contains(new Point((int)UWB_UI_Mouse.Position.X, (int)UWB_UI_Mouse.Position.Y))
                    && (Math.Abs(selectorDelta) <= m_SelectorDrawArea.Width / 2) && (m_Value > m_MinValue && m_Value < m_MaxValue))
                {
                    m_SelectorDrawArea.X = (int)(mousePosition.X - (SLIDERBAR_SELECTOR_WIDTH / 2) - selectorDelta);
                    m_PrevDelta = selectorDelta;
                    m_SelectorFreeMovement = true;
                }
                else
                {
                    m_SelectorDrawArea.X = (m_MinValue != m_MaxValue)
                        ? Convert.ToInt32((m_Value - m_MinValue) / (float)(m_MaxValue - m_MinValue) * m_Width + m_BarDrawArea.X - (SLIDERBAR_SELECTOR_WIDTH / 2))
                        : Convert.ToInt32(m_BarDrawArea.X + (m_Width / 2) - (SLIDERBAR_SELECTOR_WIDTH / 2));
                }

                spriteBatch.Draw(BAR_IMAGE, m_BarDrawArea, m_BarImageRect, m_TintColor);
                spriteBatch.Draw(SELECTOR_IMAGE, m_SelectorDrawArea, m_SelectorImageRect, m_TintColor);
            }
            base.ExecuteDrawListeners();
        }

        /// <summary>
        /// Enables or disables slider bar control
        /// If MinValue == MaxValue, then slider bar will remain disabled (must change MinValue or MaxValue first)
        /// </summary>
        public bool Disable
        {
            get { return m_Disabled; }
            set
            {
                if (m_MinValue == m_MaxValue)
                    m_Disabled = true;
                else
                    m_Disabled = value;
            }
        }
    }
}
