using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

namespace UWB_GUI
{
    public class UWB_UI_TreeView : UWB_UI_Element
    {
        private UWB_UI_TreeViewNode m_Root;
        private UWB_UI_Container m_Container;
        private SpriteFont m_SpriteFont;
        private Vector2 m_Position;
        private UWB_UI_TreeViewNode m_CurrentNode;

        public UWB_UI_TreeView(String key, Object value, UWB_UI_Container container, SpriteFont spriteFont)
        {
            m_Root = new UWB_UI_TreeViewNode(key, value, spriteFont, container);
            m_Container = container;
            m_SpriteFont = spriteFont;
            m_Position = new Vector2();
            m_CurrentNode = null;
        }

        public UWB_UI_TreeViewNode GetCurrentSelectedNode()
        {
            return m_CurrentNode;
        }

        public Object GetCurrentSelectedNodeValue()
        {
            if (m_CurrentNode != null)
            {
                return m_CurrentNode.Value;
            }
            return null;
        }

        public Vector2 Position
        {
            get { return m_Position; }
            set { m_Position = value; }
        }

        public UWB_UI_Container Container
        {
            get { return m_Container; }
            set { m_Container = value; }
        }

        public SpriteFont SpriteFont
        {
            get { return m_SpriteFont; }
            set { m_SpriteFont = value; }
        }

        public void AddChild(UWB_UI_TreeViewNode node)
        {
            m_Root.AddChild(node);
        }

        public void AddChild(String key, Object value)
        {
            m_Root.AddChild(new UWB_UI_TreeViewNode(key, value, m_SpriteFont, m_Container));
        }

        public void AddChild(String parent, String key, Object value)
        {
            UWB_UI_TreeViewNode parentNode = GetChild(parent);
            if (parentNode != null)
            {
                parentNode.AddChild(key, value, m_SpriteFont);
            }
        }

        public UWB_UI_TreeViewNode GetChild(String key)
        {
            return m_Root.GetChild(key);
        }

        public UWB_UI_TreeViewNode GetRoot()
        {
            return m_Root;
        }

        public void SetSelectedNode(String key)
        {
            UWB_UI_TreeViewNode temp = m_Root.SetSelectedNode(key);
            if (temp != null && m_CurrentNode != temp)
            {
                m_CurrentNode = temp;
                ExecuteCheckedChangeListeners();
                ExecuteValueChangedListeners();
            }
        }

        public override void SetSelected(Point mousePosition)
        {
            UWB_UI_TreeViewNode node = m_Root.SetSelected(mousePosition);
            if (node != null && m_CurrentNode != node)
            {
                if (m_CurrentNode != null)
                {
                    m_CurrentNode.IsSelected = false;
                }
                m_CurrentNode = node;
                ExecuteCheckedChangeListeners();
                ExecuteValueChangedListeners();
            }
        }

        public override void Update(GameTime gameTime)
        {
            Vector2 tempVect = new Vector2(m_Position.X, m_Position.Y);
            m_Root.Update(ref tempVect);
            ExecuteUpdateListeners();
        }

        public override void Draw(GraphicsDevice graphics, SpriteBatch spriteBatch)
        {
            m_Root.Draw(graphics, spriteBatch);
            ExecuteDrawListeners();
        }
    }
}
