using System;
using System.Collections.Generic;
using System.Collections;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

namespace UWB_GUI
{
    public class UWB_UI_TreeViewNode
    {
        private static readonly String CONTENT_TREE_VIEW_PATH_NAME = "Images/TreeView";
        private static Texture2D TREE_VIEW_IMAGE;
        private static readonly int TREE_VIEW_WIDTH = 9;
        private static readonly int TREE_VIEW_HEIGHT = 9;

        private Rectangle m_SourceImageRect;
        private Rectangle m_DrawArea;
        private List<UWB_UI_TreeViewNode> m_Children;
        private String m_Key;
        private UWB_UI_Label m_Label;
        private bool m_IsCollapsed;
        private int m_CollaspImageToLabelSpacing;
        private int m_VerticalTextSpacing;
        private int m_ChildrenHorizontalOffset;
        private Vector2 m_Position;
        private float m_ChildrenHeight;
        private UWB_UI_Container m_Container;
        private bool m_IsSelected;
        private Object m_Value;

        public UWB_UI_TreeViewNode(String key, Object value, SpriteFont spriteFont, UWB_UI_Container container)
        {
            m_SourceImageRect = new Rectangle(0, 0, TREE_VIEW_WIDTH, TREE_VIEW_HEIGHT);
            m_DrawArea = new Rectangle(0, 0, TREE_VIEW_WIDTH, TREE_VIEW_HEIGHT);

            m_Children = new List<UWB_UI_TreeViewNode>();
            m_Key = key;
            m_Value = value;
            m_Label = new UWB_UI_Label(key, container, spriteFont); ;
            m_VerticalTextSpacing = 3;
            m_ChildrenHorizontalOffset = 10;
            m_CollaspImageToLabelSpacing = 5;
            m_Position = new Vector2();
            m_ChildrenHeight = 0;
            m_Container = container;
            m_IsSelected = false;
        }

        public static void InitializeImage(ContentManager content)
        {
            TREE_VIEW_IMAGE = content.Load<Texture2D>(CONTENT_TREE_VIEW_PATH_NAME);
        }

        public String Key
        {
            get { return m_Key; }
            set { m_Key = value; }
        }

        public String Text
        {
            get { return m_Label.Text; }
            set { m_Label.Text = value; }
        }

        public UWB_UI_Label Label
        {
            get { return m_Label; }
            set { m_Label = value; }
        }

        public Object Value
        {
            get { return m_Value; }
            set { m_Value = value; }
        }

        public bool IsSelected
        {
            get { return m_IsSelected; }
            set
            {
                m_IsSelected = value;
                m_Label.DrawSelectedBackground = value;
            }
        }

        public Vector2 Position
        {
            get { return m_Position; }
        }

        public void setPosition(float xPos, float yPos)
        {
            m_Position.X = xPos;
            m_Position.Y = yPos;

            m_DrawArea.X = (int)m_Position.X + m_Container.X;
            float labelPosY;
            
            if (m_Label.Font.LineSpacing > TREE_VIEW_HEIGHT)
            {
                m_DrawArea.Y = (int)m_Position.Y + ((m_Label.Font.LineSpacing - TREE_VIEW_HEIGHT) / 2) + m_Container.Y;
                labelPosY = m_Position.Y;
            }
            else
            {
                m_DrawArea.Y = (int)m_Position.Y + m_Container.Y;
                labelPosY = m_Position.Y + ((TREE_VIEW_HEIGHT - m_Label.Font.LineSpacing) / 2);
            }
            m_Label.SetPosition(m_Position.X + TREE_VIEW_WIDTH + m_CollaspImageToLabelSpacing, labelPosY);
        }

        public void setPosition(Vector2 pos)
        {
            setPosition(pos.X, pos.Y);
        }

        public bool IsCollapsed
        {
            get { return m_IsCollapsed; }
            set { m_IsCollapsed = value; }
        }

        public void AddChild(String key, Object value, SpriteFont spriteFont)
        {
            UWB_UI_TreeViewNode tempNode = new UWB_UI_TreeViewNode(key, value, spriteFont, m_Container);
            m_ChildrenHeight += tempNode.getHeight();
            m_Children.Add(tempNode);
        }

        public void AddChild(UWB_UI_TreeViewNode node)
        {
            m_ChildrenHeight += node.getHeight();
            m_Children.Add(node);
        }

        public void RemoveChild(UWB_UI_TreeViewNode node)
        {
            m_Children.Remove(node);
        }

        private float getHeight()
        {
            return Math.Max(m_Label.Font.LineSpacing, m_DrawArea.Height);
        }

        private float getChildrenHeight()
        {
            return (m_ChildrenHeight + (m_Children.Count * m_VerticalTextSpacing));
        }

        public List<UWB_UI_TreeViewNode> GetChildren()
        {
            return m_Children;
        }

        public int getChildCount()
        {
            return m_Children.Count;
        }

        public UWB_UI_TreeViewNode GetChild(String key)
        {
            List<UWB_UI_TreeViewNode>.Enumerator etr = m_Children.GetEnumerator();
            while(etr.MoveNext())
            {
                if (etr.Current.Key == key)
                {
                    return etr.Current;
                }
                else
                {
                    UWB_UI_TreeViewNode temp = etr.Current.GetChild(key);
                    if (temp != null)
                    {
                        return temp;
                    }
                }
            }
            return null;
        }

        public bool Contains(String key)
        {
            return (GetChild(key) != null);
        }

        public bool Contains(UWB_UI_TreeViewNode node)
        {
            return m_Children.Contains(node);
        }

        public UWB_UI_TreeViewNode SetSelectedNode(String key)
        {
            UWB_UI_TreeViewNode returnVal = null;
            if (key.Equals(m_Key))
            {
                this.IsSelected = true;
                returnVal = this;
            }
            else
            {
                this.IsSelected = false;
            }

            List<UWB_UI_TreeViewNode>.Enumerator etr = m_Children.GetEnumerator();
            while (etr.MoveNext())
            {
                UWB_UI_TreeViewNode tempVal = etr.Current.SetSelectedNode(key);
                if (tempVal != null)
                {
                    returnVal = tempVal;
                }
            }

            return returnVal;
        }

        public UWB_UI_TreeViewNode SetSelected(Point mousePosition)
        {
            if (m_DrawArea.Contains(mousePosition))
            {
                m_IsCollapsed = !m_IsCollapsed;
                return null;
            }
            else if (m_Label.GetBounds().Contains(mousePosition))
            {
                this.IsSelected = true;
                return this;
            }
            else
            {
                if (!m_IsCollapsed)
                {
                    List<UWB_UI_TreeViewNode>.Enumerator etr = m_Children.GetEnumerator();
                    while (etr.MoveNext())
                    {
                        UWB_UI_TreeViewNode node = etr.Current.SetSelected(mousePosition);
                        if (node != null)
                        {
                            return node;
                        }
                    }
                }
               
            }
            return null;
        }

        public void Update(ref Vector2 pos)
        {
            setPosition(pos);
            pos.X += m_ChildrenHorizontalOffset;
            float tempYOffset = getHeight() + m_VerticalTextSpacing;
            pos.Y += tempYOffset;
            if (!m_IsCollapsed)
            {
                List<UWB_UI_TreeViewNode>.Enumerator etr = m_Children.GetEnumerator();
                while (etr.MoveNext())
                {
                    etr.Current.Update(ref pos);
                }
            }
            pos.X -= m_ChildrenHorizontalOffset;
        }

        public void Draw(GraphicsDevice graphics, SpriteBatch spriteBatch)
        {
            if (m_Children.Count > 0)
            {
                if (!m_IsCollapsed)
                {
                    List<UWB_UI_TreeViewNode>.Enumerator etr = m_Children.GetEnumerator();
                    while (etr.MoveNext())
                    {
                        etr.Current.Draw(graphics, spriteBatch);
                    }
                    m_SourceImageRect.X = 0;
                }
                else
                {
                    m_SourceImageRect.X = TREE_VIEW_WIDTH;
                }
                spriteBatch.Draw(TREE_VIEW_IMAGE, m_DrawArea, m_SourceImageRect, Color.White);
            }
            m_Label.Draw(graphics, spriteBatch);
        }
    }
}
