using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Storage;

namespace UWB_XNALib
{
    public class UWB_Camera
    {
        private Vector3 m_CameraPos;
        private Vector3 m_LookAt;
        private Vector3 m_UpVector;
        private float m_ViewFoV;
        private float m_ViewNearPlane, m_ViewFarPlane;
        private float m_XRotStart, m_YRotStart;
        private float m_XRot, m_YRot;
        private int m_TmpInitXPos, m_TmpInitYPos;

        public UWB_Camera()
        {
            m_CameraPos = Vector3.Forward;
            m_LookAt = Vector3.Zero;
            m_UpVector = new Vector3(0f, 1f, 0f);
            m_ViewFoV = 45.0f;
            m_ViewNearPlane = .1f;
            m_ViewFarPlane = 100f;
            m_XRotStart = 0.0f;
            m_YRotStart = 0.0f;
            m_XRot= 0.0f;
            m_YRot = 0.0f;
            m_TmpInitXPos = 0;
            m_TmpInitYPos = 0;
        }


        public void SetViewFrustumInDegree(float v, float n, float f)
        {
            m_ViewFoV = v;
            m_ViewNearPlane = n;
            m_ViewFarPlane = f;
        }

        public void GetViewFrustumInDegree(ref float v, ref float n, ref float f)
        {
            v = m_ViewFoV;
            n = m_ViewNearPlane;
            f = m_ViewFarPlane;
        }
     
        public Vector3 CameraPosition
        {
            get { return m_CameraPos; }
            set { m_CameraPos = value; }
        }

        public Vector3 CameraLookAt
        {
            get { return m_LookAt; }
            set { m_LookAt = value; }
        }

        public Vector3 CameraUp
        {
            get { return m_UpVector; }
            set { m_UpVector = value; }
        }

        public float FoV
        {
            get { return m_ViewFoV; }
            set { m_ViewFoV = value; }
        }

        public float FoVRadian
        {
            get { return MathHelper.ToRadians(m_ViewFoV); }
            set { m_ViewFoV = MathHelper.ToDegrees(value); }
        }

        public float NearClipping
        {
            get { return m_ViewNearPlane; }
            set { m_ViewNearPlane = value; }
        }

        public float FarClipping
        {
            get { return m_ViewFarPlane; }
            set { m_ViewFarPlane = value; }
        }

        public void GetMouseRotationsInRadian(ref float xRot, ref Vector3 xRef, ref float yRot, ref Vector3 yRef)
        {
          Vector3 view_direction;
          yRef =  m_UpVector;
          view_direction = m_LookAt - m_CameraPos;
          xRef =  Vector3.Cross(m_UpVector, view_direction);
          xRef.Normalize();
          yRef.X = 0.0f;
          yRef.Y = 1.0f;
          yRef.Z = 0.0f;  // sense of up is important.
          xRot = MathHelper.ToRadians(m_XRot);
          yRot = MathHelper.ToRadians(m_YRot);
        }

        public void ResetRotation()
        {
          m_XRot = 0.0f;
          m_YRot = 0.0f;
        }

        public void BeginMouseRotate(int x, int y)
        {
          m_XRotStart = m_XRot;
          m_YRotStart = m_YRot;
          m_TmpInitXPos = x;
          m_TmpInitYPos = y;
        }

        public void MouseRotate(int x, int y)
        {
          m_XRot = m_XRotStart + (y-m_TmpInitYPos);   // says each pixel movement is mapped to one degree
          m_YRot = m_YRotStart + (x-m_TmpInitXPos);
        }


        public void BeginMouseTrack(int x, int y)
        {
          m_TmpInitXPos = x;
          m_TmpInitYPos = y;
        }

        
        private static float TRACK_FACTOR  = 0.1f;
        
        public void MouseTrack(int x, int y)
        {
          float dx = (float)(x-m_TmpInitXPos) * TRACK_FACTOR;     // each pixel tracks 0.2 WC unit
          float dy = (float)(y-m_TmpInitYPos) * TRACK_FACTOR;

          Vector3 view_direction, cross_direction;
          Vector3 up_direction = m_UpVector;
          view_direction = m_LookAt - m_CameraPos;

          cross_direction = Vector3.Cross(up_direction, view_direction);
          cross_direction.Normalize();

          up_direction = Vector3.Cross(view_direction, cross_direction);
          up_direction.Normalize();

          Vector3 delta = (-dx * cross_direction) + (dy * up_direction);
          m_CameraPos = m_CameraPos - delta;
          m_LookAt = m_LookAt - delta;

          // continue to track
          BeginMouseTrack(x, y);
        }

        public void BeginMouseZoom(int x)
        {
          m_TmpInitXPos = x;
        }

        public void MouseZoom(int x)
        {
          float dx = (float)(x - m_TmpInitXPos) * TRACK_FACTOR;
          Vector3 view_vec = m_LookAt - m_CameraPos;
          float view_distance = view_vec.Length();
          view_vec.Normalize();
          m_CameraPos = m_LookAt - ( (view_distance + dx) * view_vec);
          BeginMouseZoom(x);
        }


    }
}


