using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using System.Diagnostics;

namespace UWB_XNALib
{
    public class UWB_XNADrawHelper : UWB_DrawHelper
    {

        public UWB_XNADrawHelper()
        {

        }

        public override bool drawPoint(Vector3 position)
        {
            GraphicsDevice device = XNAGraphicsDevice.m_TheAPI.GraphicsDevice;
            Debug.Assert(device != null, "device not valid on DrawHelper drawCircle call");
            device.VertexDeclaration = XNAGraphicsDevice.m_TheAPI.VertexDec;
            DeviceVertexFormat[] v = new DeviceVertexFormat[1];

            v[0].Position = position;
            v[0].Color = mColor1;
            v[0].U = 0.0f;
            v[0].V = 0.0f;
            v[0].Normal = Vector3.UnitY;

            float pointSize = device.RenderState.PointSize;
            device.RenderState.PointSize = mPointSize;
            device.RenderState.PointSpriteEnable = true;
            device.DrawUserPrimitives(PrimitiveType.PointList, v, 0, 1);

            return true;
        }

        public override bool drawLine(Vector3 start, Vector3 end)
        {
            GraphicsDevice device = XNAGraphicsDevice.m_TheAPI.GraphicsDevice;
            Debug.Assert(device != null, "device not valid on DrawHelper drawCircle call");
            device.VertexDeclaration = XNAGraphicsDevice.m_TheAPI.VertexDec;
            DeviceVertexFormat[] v = new DeviceVertexFormat[2];

            Color flatColor = mColor1;

            Vector3 dir = end - start;
            Vector3 normal = Vector3.UnitX;

            if ((dir.X > 0.01f) && (dir.Y > 0.01f))
            {
                normal.X = -dir.Y;
                normal.Y = dir.X;
                normal.Z = 0.0f; // a normal to the line direction
            }

            v[0].Position = start;
            v[0].Color = flatColor;
            v[0].U = 0.0f;
            v[0].V = 0.0f;
            v[0].Normal = normal;

            v[1].Position = end;
            v[1].U = 1.0f;
            v[1].V = 1.0f;
            v[1].Normal = normal;

            if (mShadeMode == eShadeMode.smFlat)
                v[1].Color = flatColor;
            else
                v[1].Color = mColor2;

            UWB_XNAEffect effect = XNAGraphicsDevice.m_TheAPI.LightingEffect;

            Texture2D texture = null;

            if (m_bTexturingEnabled && m_TexFileName != "")
                texture = XNAGraphicsDevice.m_TheAPI.RetrieveTexture(m_TexFileName);

            effect.Material = this.m_Material;

            if (texture != null)
            {
                effect.Texture = texture;
                effect.TextureEnabled = true;
            }
            else
            {
                effect.Texture = texture;
                effect.TextureEnabled = false;
            }

            effect.Begin();

            foreach (EffectPass pass in effect.CurrentTechnique.Passes)
            {
                pass.Begin();
                device.DrawUserPrimitives<DeviceVertexFormat>(PrimitiveType.LineList, v, 0, 1);
                pass.End();
            }

            effect.End();

            return true;
        }

        public override bool drawCircle(Vector3 center, float radius)
        {
            GraphicsDevice device = XNAGraphicsDevice.m_TheAPI.GraphicsDevice;
            Debug.Assert(device != null, "device not valid on DrawHelper drawCircle call");
            device.VertexDeclaration = XNAGraphicsDevice.m_TheAPI.VertexDec;

            if (radius > 0)
            {
                int kNumPts = 40;
                if (mLod == eLevelofDetail.lodLow)
                    kNumPts = 10;
                else if (mLod == eLevelofDetail.lodMed)
                    kNumPts = 20;
                DeviceVertexFormat[] v = new DeviceVertexFormat[kNumPts];

                float delta = (float)(UWB_Math.UWB_ToRadians((float)360 / (float)(kNumPts - 2)));

                Color flatColor = mColor1;

                v[0].Position = new Vector3(center.X, center.Y, center.Z);
                v[0].Color = flatColor;
                v[0].U = 0.5f;
                v[0].V = 0.5f;
                v[0].Normal = Vector3.UnitZ;

                for (int i = 0; i < (kNumPts - 2); i++)
                {
                    float X = center.X + (radius * (float)Math.Cos(i * delta));
                    float Y = center.Y + (radius * (float)Math.Sin(i * delta));
                    v[i + 1].Color = flatColor;
                    v[i + 1].Position = new Vector3(X, Y, center.Z);
                    v[i + 1].U = 0.5f + (0.5f * (float)Math.Cos(i * delta));
                    v[i + 1].V = 0.5f - (0.5f * (float)Math.Sin(i * delta));
                    v[i + 1].Normal = Vector3.UnitZ;
                }
                v[kNumPts - 1] = v[1];

                if (mShadeMode == eShadeMode.smGouraud)
                    v[0].Color = mColor2;

                UWB_XNAEffect effect = XNAGraphicsDevice.m_TheAPI.LightingEffect;

                Texture2D texture = null;

                if (m_bTexturingEnabled && m_TexFileName != "")
                    texture = XNAGraphicsDevice.m_TheAPI.RetrieveTexture(m_TexFileName);

                effect.Material = this.m_Material;

                if (texture != null)
                {
                    effect.Texture = texture;
                    effect.TextureEnabled = true;
                }
                else
                {
                    effect.Texture = texture;
                    effect.TextureEnabled = false;
                }

                effect.Begin();

                foreach (EffectPass pass in effect.CurrentTechnique.Passes)
                {
                    pass.Begin();
                    device.DrawUserPrimitives<DeviceVertexFormat>(PrimitiveType.TriangleFan, v, 0, (kNumPts - 2));
                    pass.End();
                }

                effect.End();
            }

            return true;

        }

        public override bool drawRectangle(Vector3 min, Vector3 max)
        {
            GraphicsDevice device = XNAGraphicsDevice.m_TheAPI.GraphicsDevice;
            Debug.Assert(device != null, "device not valid on DrawHelper drawRectangle call");
            device.VertexDeclaration = XNAGraphicsDevice.m_TheAPI.VertexDec;
            DeviceVertexFormat[] v = new DeviceVertexFormat[6];
            Color flatColor = mColor1;

            v[0].Position = Vector3.Divide((min + max), 2);
            v[0].Color = flatColor;
            v[0].U = 0.5f;
            v[0].V = 0.5f;
            v[0].Normal = Vector3.UnitZ;

            v[1].Position = new Vector3(min.X, max.Y, max.Z);
            v[1].Color = flatColor;
            v[1].U = 1.0f;
            v[1].V = 0.0f;
            v[1].Normal = Vector3.UnitZ;

            v[2].Position = min;
            v[2].Color = flatColor;
            v[2].U = 1.0f;
            v[2].V = 1.0f;
            v[2].Normal = Vector3.UnitZ;

            v[3].Position = new Vector3(max.X, min.Y, min.Z);
            v[3].Color = flatColor;
            v[3].U = 0.0f;
            v[3].V = 1.0f;
            v[3].Normal = Vector3.UnitZ;

            v[4].Position = max;
            v[4].Color = flatColor;
            v[4].U = 0.0f;
            v[4].V = 0.0f;
            v[4].Normal = Vector3.UnitZ;

            v[5] = v[1];

            if (mShadeMode == eShadeMode.smGouraud)
                v[0].Color = mColor2;

            UWB_XNAEffect effect = XNAGraphicsDevice.m_TheAPI.LightingEffect;

            Texture2D texture = null;

            if (m_bTexturingEnabled && m_TexFileName != "")
                texture = XNAGraphicsDevice.m_TheAPI.RetrieveTexture(m_TexFileName);

            effect.Material = this.m_Material;

            if (texture != null)
            {
                effect.Texture = texture;
                effect.TextureEnabled = true;
            }
            else
            {
                effect.Texture = texture;
                effect.TextureEnabled = false;
            }

            effect.Begin();

            foreach (EffectPass pass in effect.CurrentTechnique.Passes)
            {
                pass.Begin();
                device.DrawUserPrimitives<DeviceVertexFormat>(PrimitiveType.TriangleFan, v, 0, 4);
                pass.End();
            }

            effect.End();

            return true;
        }

        public override bool drawRectangleXZ(Vector3 min, Vector3 max)
        {
            GraphicsDevice device = XNAGraphicsDevice.m_TheAPI.GraphicsDevice;
            Debug.Assert(device != null, "device not valid on DrawHelper drawRectangle call");
            device.VertexDeclaration = XNAGraphicsDevice.m_TheAPI.VertexDec;
            DeviceVertexFormat[] v = new DeviceVertexFormat[6];
            Color flatColor = mColor1;

            v[0].Position = Vector3.Divide((min + max), 2);
            v[0].Color = flatColor;
            v[0].U = 0.5f;
            v[0].V = 0.5f;
            v[0].Normal = Vector3.UnitY;

            v[1].Position = new Vector3(min.X, max.Y, max.Z);
            v[1].Color = flatColor;
            v[1].U = 1.0f;
            v[1].V = 0.0f;
            v[1].Normal = Vector3.UnitY;

            v[2].Position = min;
            v[2].Color = flatColor;
            v[2].U = 1.0f;
            v[2].V = 1.0f;
            v[2].Normal = Vector3.UnitY;

            v[3].Position = new Vector3(max.X, min.Y, min.Z);
            v[3].Color = flatColor;
            v[3].U = 0.0f;
            v[3].V = 1.0f;
            v[3].Normal = Vector3.UnitY;

            v[4].Position = max;
            v[4].Color = flatColor;
            v[4].U = 0.0f;
            v[4].V = 0.0f;
            v[4].Normal = Vector3.UnitY;

            v[5] = v[1];

            if (mShadeMode == eShadeMode.smGouraud)
                v[0].Color = mColor2;

            UWB_XNAEffect effect = XNAGraphicsDevice.m_TheAPI.LightingEffect;
            effect.Material = this.m_Material;

            Texture2D texture = null;

            if (m_bTexturingEnabled && m_TexFileName != "")
                texture = XNAGraphicsDevice.m_TheAPI.RetrieveTexture(m_TexFileName);

            if (texture != null)
            {
                effect.Texture = texture;
                effect.TextureEnabled = true;
            }
            else
            {
                effect.Texture = texture;
                effect.TextureEnabled = false;
            }

            effect.Begin();

            foreach (EffectPass pass in effect.CurrentTechnique.Passes)
            {
                pass.Begin();

                device.DrawUserPrimitives<DeviceVertexFormat>(PrimitiveType.TriangleFan, v, 0, 4);

                pass.End();
            }

            effect.End();

            return true;
        }


        public override void setFillMode(eFillMode fillMode)
        {
            GraphicsDevice graphicsDevice = XNAGraphicsDevice.m_TheAPI.GraphicsDevice;

            if (graphicsDevice != null)
            {
                switch (fillMode)
                {
                    case eFillMode.fmSolid:
                        graphicsDevice.RenderState.FillMode = FillMode.Solid;
                        break;
                    case eFillMode.fmPoint:
                        graphicsDevice.RenderState.FillMode = FillMode.Point;
                        break;
                    case eFillMode.fmWireframe:
                        graphicsDevice.RenderState.FillMode = FillMode.WireFrame;
                        break;
                }
            }
        }

        public override bool accumulateModelTransform(
            Vector3 translation,
            Vector3 scale,
            float rotationRadians,
            Vector3 rotation_axis,
            Vector3 pivot)
        {
            if (XNAGraphicsDevice.m_TheAPI.GraphicsDevice == null)
                return false;

            Vector3 axis = new Vector3(rotation_axis.X, rotation_axis.Y, rotation_axis.Z);
            m_MatrixStack.TranslateLocal(translation);
            m_MatrixStack.TranslateLocal(pivot);
            m_MatrixStack.RotateAxisLocal(axis, rotationRadians);
            m_MatrixStack.ScaleLocal(scale);
            m_MatrixStack.TranslateLocal(-pivot);

            XNAGraphicsDevice.m_TheAPI.WorldMatrix = m_MatrixStack.Top();
            return true;

        }

        public override bool accumulateModelTransform(
            Vector3 translation,
            Vector3 scale,
            Vector3 rotationRadians,
            Vector3 pivot)
        {
            if (XNAGraphicsDevice.m_TheAPI.GraphicsDevice == null)
                return false;

            Vector3 z_axis = new Vector3(0, 0, 1);
            Vector3 y_axis = new Vector3(0, 1, 0);
            Vector3 x_axis = new Vector3(1, 0, 0);
            m_MatrixStack.TranslateLocal(translation);
            m_MatrixStack.TranslateLocal(pivot);
            m_MatrixStack.RotateAxisLocal(x_axis, rotationRadians.X);
            m_MatrixStack.RotateAxisLocal(y_axis, rotationRadians.Y);
            m_MatrixStack.RotateAxisLocal(z_axis, rotationRadians.Z);
            m_MatrixStack.ScaleLocal(scale);
            m_MatrixStack.TranslateLocal(-pivot);

            XNAGraphicsDevice.m_TheAPI.WorldMatrix = m_MatrixStack.Top();
            return true;

        }

        public override bool accumulateModelTransform(
           Vector3 translation,
           Vector3 scale,
           Matrix rotation,
           Vector3 pivot)
        {
            if (XNAGraphicsDevice.m_TheAPI.GraphicsDevice == null)
                return false;

            m_MatrixStack.TranslateLocal(translation);
            m_MatrixStack.TranslateLocal(pivot);
            m_MatrixStack.MultMatrixLocal(rotation);
            m_MatrixStack.ScaleLocal(scale);
            m_MatrixStack.TranslateLocal(-pivot);

            XNAGraphicsDevice.m_TheAPI.WorldMatrix = m_MatrixStack.Top();
            return true;
        }

        public override bool pushModelTransform()
        {
            if (XNAGraphicsDevice.m_TheAPI.GraphicsDevice == null)
                return false;

            m_MatrixStack.Push();
            XNAGraphicsDevice.m_TheAPI.WorldMatrix = m_MatrixStack.Top();
            return true;
        }

        public override bool popModelTransform()
        {
            if (XNAGraphicsDevice.m_TheAPI.GraphicsDevice == null)
                return false;

            m_MatrixStack.Pop();
            XNAGraphicsDevice.m_TheAPI.WorldMatrix = m_MatrixStack.Top();
            return true;
        }

        public override bool initializeModelTransform()
        {
            if (XNAGraphicsDevice.m_TheAPI.GraphicsDevice == null)
                return false;

            m_MatrixStack.LoadIdentity();
            XNAGraphicsDevice.m_TheAPI.WorldMatrix = m_MatrixStack.Top();
            return true;
        }
        public bool TransformPoint(ref Vector3 point)
        {
            Vector3 pt = new Vector3(point.X, point.Y, point.Z);

            Matrix m = m_MatrixStack.Top();
            point.X = pt.X * m.M11 + pt.Y * m.M21 + pt.Z * m.M31 + m.M41;
            point.Y = pt.X * m.M12 + pt.Y * m.M22 + pt.Z * m.M32 + m.M42;
            point.Z = pt.X * m.M13 + pt.Y * m.M23 + pt.Z * m.M33 + m.M43;
            return true;
        }

        public override bool EnableBlending(bool on)
        {
            GraphicsDevice d = XNAGraphicsDevice.m_TheAPI.GraphicsDevice;
            if (d != null)
            {
                m_bBlendingEnabled = on;

                if (on)
                {
                    d.RenderState.AlphaBlendEnable = true;
                    d.RenderState.SourceBlend = Blend.SourceAlpha;
                    d.RenderState.DestinationBlend = Blend.InverseSourceAlpha;
                    d.RenderState.BlendFunction = BlendFunction.Add;
                    d.RenderState.AlphaTestEnable = true;
                }
                else
                {
                    d.RenderState.AlphaBlendEnable = false;
                }
            }
            return true;
        }

        // Pointless funtion at the moment!!
        public override bool EnableTexture(bool on)
        {
            m_bTexturingEnabled = on;
            return !on;
        }


        // Pointless funtion at the moment!!
        public override bool EnableLighting(bool on)
        {
            m_bLightingEnabled = on;
            return !on;
        }

    }
}
