using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using UWB_GUI;
using UWB_XNALib;
using XNA_UI_Controls;

namespace XNAMultipleLights
{
    public class DrawAndMouseHandler : DrawOnlyHandler
    {
        DrawOnlyHandler m_LinkedHandler;

        public DrawAndMouseHandler(Rectangle deviceBounds, Rectangle worldBounds, Model model, XNA_UI_GUI gui)
            : base(deviceBounds, worldBounds, model, gui) { }

        public DrawAndMouseHandler(Rectangle deviceBounds, Model model, XNA_UI_GUI gui)
            : base(deviceBounds, model.WorldBounds, model, gui) { }

        public override void OnMouseButton(bool down, int hwX, int hwY)
        {
            int deviceX = 0, deviceY = 0;
            HardwareToDevice(hwX, hwY, out deviceX, out deviceY);
            String msg = "";

            if (down)
            {

                if (UWB_UI_Mouse.LeftMouseButtonDown)
                {
                    //left (A) button is down
                    m_Camera.BeginMouseRotate(deviceX, deviceY);
                    msg = String.Format("Begin Camera Rotation: ({0} {1})", deviceX, deviceY);
                }
                else if (UWB_UI_Mouse.MiddleMouseButtonDown)
                {
                    //middle (Y) button is down
                    m_Camera.BeginMouseTrack(deviceX, deviceY);
                    msg = String.Format("Begin Camera Tracking: ({0} {1})", deviceX, deviceY);
                }
                else if (UWB_UI_Mouse.RightMouseButtonDown)
                {
                    //right (B) button is down
                    m_Camera.BeginMouseZoom(deviceX);
                    msg = String.Format("Begin Camera Zooming: ({0})", deviceX);
                }
            }

            UWB_XNALib.UWB_Utility.echoToStatusArea(msg);
            
        }

        public override void OnMouseMove(bool down, int hwX, int hwY)
        {
            int deviceX = 0, deviceY = 0;
            HardwareToDevice(hwX, hwY, out deviceX, out deviceY);
            String msg = "";

            if (down)
            {
                if (UWB_UI_Mouse.LeftMouseButtonDown)
                {
                    //left (A) button is down
                    m_Camera.MouseRotate(deviceX, deviceY);
                    msg = String.Format("Camera Rotation To: ({0} {1})", deviceX, deviceY);
                }
                else if (UWB_UI_Mouse.MiddleMouseButtonDown)
                {
                    //middle (Y) button is down
                    m_Camera.MouseTrack(deviceX, deviceY);
                    msg = String.Format("Camera Tracking To: ({0} {1})", deviceX, deviceY);
                }
                else if (UWB_UI_Mouse.RightMouseButtonDown)
                {
                    //right (B) button is down
                    m_Camera.MouseZoom(deviceX);
                    msg = String.Format("Camera Zoom: ({0})", deviceX);
                }
            }

            UWB_XNALib.UWB_Utility.echoToStatusArea(msg);
        }

        public override void DrawGraphics()
        {
            if (m_bVisible)
            {
                LoadW2NDCXform();
                LoadViewXform();
                LoadProjectionXform();

                BeginDraw();

                m_Model.DrawModel();

                UWB_XNADrawHelper helper = new UWB_XNADrawHelper();
                helper.initializeModelTransform();
				if(m_LinkedHandler != null)
					m_LinkedHandler.DrawWCWindow(helper);

                EndDraw();
            }
        }

        public void SetLinkedHandler(DrawOnlyHandler handler)
        {
            m_LinkedHandler = handler;
        }

        public void ComputeBoundPosition(float x, float y)
        {
            // Sets the m_small_view's center position to current mouse x/y position ...
            UWB_BoundingBox b = m_LinkedHandler.WCWindow;
            float h = b.height() / 2.0f;
            float w = b.width() / 2.0f;
            b.setCorners(new Vector3(x - w, y - h, 0.0f), new Vector3(x + w, y + h, 0.0f));
            m_LinkedHandler.WCWindow = b;
        }

        private void ComputeBoundPosition(Vector2 mousePos)
        {
            ComputeBoundPosition(mousePos.X, mousePos.Y);
        }

    }
}
