using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

using UWB_XNALib;
using XNA_UI_Controls;
using UWB_GUI;

namespace XNAMultipleLights
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        private Model m_Model;
        private DrawAndMouseHandler m_MainView;
        private DrawOnlyHandler m_SmallView;
        private XNA_UI_GUI m_GUI;
        private RGameView rgv;
        private RGraphicsDeviceService rgds;

#if XBOX360
        public const int displayWidth = 1280;
        public const int displayHeight = 768;
#else
        public const int displayWidth = 1000; //821;
        public const int displayHeight = SceneTreeXformInfoPrimitiveControl.height;//462;
#endif

        public Game1()
        {
            XNAGraphicsDevice.m_TheAPI.CreateGraphicsContext(this, displayWidth, displayHeight);
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            XNAGraphicsDevice.m_TheAPI.Initialize();

            m_Model = new Model();
            InitializeGUI();

            Rectangle deviceBounds = new Rectangle(0, 0, displayWidth, displayHeight);
            m_MainView = new DrawAndMouseHandler(deviceBounds, m_Model, m_GUI);

            deviceBounds.Width = deviceBounds.Height = 200;
            deviceBounds.X = displayWidth - SceneTreeControl.width - 10 - deviceBounds.Width;
            m_SmallView = new DrawOnlyHandler(deviceBounds, m_Model, m_GUI);
            m_SmallView.BackgroundColor = Color.LightSteelBlue;

            m_MainView.GetCamera().CameraLookAt = Vector3.Zero;
            m_MainView.GetCamera().CameraPosition = new Vector3(-10, 10, -25);
            m_MainView.GetCamera().FoV = (float)Math.PI / 4f;
            m_MainView.GetCamera().NearClipping = 0.5f;
            m_MainView.GetCamera().FarClipping = 100.0f;

            m_SmallView.GetCamera().CameraLookAt = Vector3.Zero;
            m_SmallView.GetCamera().CameraPosition = new Vector3(30, 30, 30);
            m_SmallView.GetCamera().FoV = (float)Math.PI / 4f;
            m_SmallView.GetCamera().NearClipping = 0.5f;
            m_SmallView.GetCamera().FarClipping = 150.0f;

            rgv = new RGameView();
            rgds = new RGraphicsDeviceService();
            rgds.GraphicsDevice = XNAGraphicsDevice.m_TheAPI.GraphicsDevice;
            rgv.SetService(rgds);
            XNAGraphicsDevice.m_TheAPI.SetService(rgv);
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            m_Model.UpdateSimulation();

            m_GUI.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            XNAGraphicsDevice.m_TheAPI.Clear();

            m_MainView.DrawGraphics();
            //m_SmallView.DrawGraphics();
            m_GUI.Draw();
        }

        private void InitializeGUI()
        {
            m_GUI = new XNA_UI_GUI(this);

            SceneTreeXformInfoPrimitiveControl stxfpc = new SceneTreeXformInfoPrimitiveControl(new Point(displayWidth - SceneTreeXformInfoPrimitiveControl.width, 0), m_Model.GetRootNode(), m_GUI);
            stxfpc.TranslateRange = new Vector2(-10f, 10f);
            stxfpc.PivotRange = new Vector2(-10f, 10f);

            EchoStatusBar status = new EchoStatusBar(new Point(10, displayHeight - EchoStatusBar.height - 10), m_GUI);
        }
    }
}
