using System;
using System.Collections.Generic;
using System.Text;
using UWB_XNALib;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace XNAMultipleLights
{
    public class Model
    {
		/// size of the world
		private UWB_BoundingBox m_WorldBounds = new UWB_BoundingBox();

		///
		/// to keep track of elapses in between updates
		private UWB_Clock m_stop_watch = new UWB_Clock();

		private UWB_SceneNode m_RootNode;
		private UWB_XNADrawHelper m_DrawHelper = new UWB_XNADrawHelper();

		private UWB_SceneNode m_pActiveNode;
		private bool m_bActiveNodeIsPicked;

		public Model()
		{
			m_DrawHelper.initializeModelTransform();

			m_RootNode = new UWB_SceneNode("Scene");
			m_bActiveNodeIsPicked = false;

            UWB_XNAPrimitiveMesh tigerMesh = new UWB_XNAPrimitiveMesh("tiger");
            tigerMesh.EnableTexturing(true);
            UWB_SceneNode pTigerMeshNode = new UWB_SceneNode("tiger mesh");
            pTigerMeshNode.setPrimitive(tigerMesh);
            UWB_XFormInfo xf = pTigerMeshNode.getXFormInfo();
            xf.SetTranslation(new Vector3(-3.0f, 2f, 4f));
            pTigerMeshNode.setXFormInfo(xf);

            UWB_SceneNode pFloorMeshNode = new UWB_SceneNode("floor mesh");
            UWB_XNAPrimitiveMesh m_FloorMesh = new UWB_XNAPrimitiveMesh("HiResFloor");
            //m_FloorMesh.EnableTexturing(true);
            //m_FloorMesh.SetTextureFileName("Building1");
            pFloorMeshNode.setPrimitive(m_FloorMesh);
            xf = pFloorMeshNode.getXFormInfo();
            xf.SetScale(new Vector3(10.0f, 1.0f, 10.0f));
            xf.SetTranslation(new Vector3(0.0f, 0.0f, 0.0f));
            pFloorMeshNode.setXFormInfo(xf);
            
            UWB_SceneNode pRectNode = new UWB_SceneNode("rect");
            UWB_PrimitiveRectangleXZ pRec = new UWB_PrimitiveRectangleXZ();
            //pRec.EnableTexturing(true);
            //pRec.EnableBlending(true);
            //pRec.SetTextureFileName("Building1");
            pRec.setCorners(new Vector3(2, 0, 2), new Vector3(-2, 0, -2));
            UWB_Material m = pRec.Material;
            m.Diffuse = new Vector4(0.8f, 0.8f, 0.8f, 1.0f);
            m.Ambient = new Vector4(1.0f, 0.0f, 1.0f, 1.0f);
            m.Power = 1.0f;
            pRec.Material = m;
            pRectNode.setPrimitive(pRec);

            xf = pRectNode.getXFormInfo();
            xf.SetTranslation(new Vector3(3.0f, 2.0f, -3.0f));
            pRectNode.setXFormInfo(xf);

            UWB_SceneNode pAxisNode = new UWB_SceneNode("axis frame");
            xf = pAxisNode.getXFormInfo();
            xf.SetScale(new Vector3(3.0f, 3.0f, 3.0f));
            pAxisNode.setXFormInfo(xf);
            UWB_XNAPrimitiveMeshAxis pMeshAxis = new UWB_XNAPrimitiveMeshAxis();
            //pMeshAxis.EnableBlending(true);
            pAxisNode.setPrimitive(pMeshAxis);

            m_RootNode.insertChildNode(pTigerMeshNode);
            m_RootNode.insertChildNode(pRectNode);
            m_RootNode.insertChildNode(pAxisNode);
            m_RootNode.insertChildNode(pFloorMeshNode);

			m_pActiveNode = m_RootNode;
			m_WorldBounds.setCorners( new Vector3(-10,-10,0), new Vector3(10,10,0) );

            
            // Setup lights
            UWB_XNAPointLight pointLight = XNAGraphicsDevice.m_TheAPI.LightManager.CreatePointLight();
            pointLight.Ambient = new Vector4(0.1f, 0.1f, 0.1f, 1.0f);
            pointLight.Diffuse = new Vector4(0.2f, 0.2f, 0.2f, 1.0f);
            pointLight.Specular = new Vector4(0.2f, 0.2f, 0.2f, 1.0f);
            pointLight.Direction = new Vector3(0.0f, -1.0f, 0.0f);
            pointLight.Position = new Vector3(2.0f, 12.0f, 2.0f);
            pointLight.Color = Color.Blue.ToVector4();
            pointLight.Attenuation = new Vector3(1.0f, 0.0f, 0.0f);
            pointLight.SwitchOnLight();

            UWB_XNADirectionalLight dirLight = XNAGraphicsDevice.m_TheAPI.LightManager.CreateDirectionalLight();
            dirLight.Ambient = new Vector4(0.2f, 0.1f, 0.2f, 1.0f);
            dirLight.Diffuse = new Vector4(0.1f, 0.1f, 0.1f, 1.0f);
            dirLight.Specular = new Vector4(0.2f, 0.2f, 0.2f, 1.0f);
            dirLight.Direction = new Vector3(0.0f, -1.0f, 0.0f);
            dirLight.Position = new Vector3(0.0f, 12.0f, 0.0f);
            dirLight.Color = Color.Blue.ToVector4();
            dirLight.SwitchOnLight();

            UWB_XNASpotLight spotLight1 = XNAGraphicsDevice.m_TheAPI.LightManager.CreateSpotLight();
            spotLight1.Ambient = new Vector4(0.1f, 0.1f, 0.1f, 1.0f);
            spotLight1.Diffuse = new Vector4(0.5f, 0.5f, 0.5f, 1.0f);
            spotLight1.Specular = new Vector4(0.2f, 0.2f, 0.2f, 1.0f);
            spotLight1.Direction = new Vector3(0.0f, -1.0f, 0.0f);
            spotLight1.Position = new Vector3(0.0f, 10.0f, 0.0f);
            spotLight1.Color = Color.Red.ToVector4();
            spotLight1.Attenuation = new Vector3(1.0f, 0.0f, 0.0f);
            spotLight1.Theta = MathHelper.ToRadians(10.0f);
            spotLight1.Phi = MathHelper.ToRadians(20.0f);
            spotLight1.SwitchOnLight();

            UWB_XNASpotLight spotLight2 = XNAGraphicsDevice.m_TheAPI.LightManager.CreateSpotLight();
            spotLight2.Ambient = new Vector4(0.1f, 0.1f, 0.1f, 1.0f);
            spotLight2.Diffuse = new Vector4(0.5f, 0.5f, 0.0f, 1.0f);
            spotLight2.Specular = new Vector4(0.2f, 0.2f, 0.2f, 1.0f);
            spotLight2.Direction = new Vector3(0.0f, -1.0f, 0.0f);
            spotLight2.Position = new Vector3(2.0f, 8.0f, 2.0f);
            spotLight2.Color = Color.Red.ToVector4();
            spotLight2.Attenuation = new Vector3(1.0f, 0.0f, 0.0f);
            spotLight2.Theta = MathHelper.ToRadians(10.0f);
            spotLight2.Phi = MathHelper.ToRadians(20.0f);
            spotLight2.SwitchOnLight();

            UWB_XNASpotLight spotLight3 = XNAGraphicsDevice.m_TheAPI.LightManager.CreateSpotLight();
            spotLight3.Ambient = new Vector4(0.1f, 0.1f, 0.1f, 1.0f);
            spotLight3.Diffuse = new Vector4(0.0f, 0.0f, 0.5f, 1.0f);
            spotLight3.Specular = new Vector4(0.2f, 0.2f, 0.2f, 1.0f);
            spotLight3.Direction = new Vector3(0.0f, -1.0f, 0.0f);
            spotLight3.Position = new Vector3(2.0f, 8.0f, -2.0f);
            spotLight3.Color = Color.Red.ToVector4();
            spotLight3.Attenuation = new Vector3(1.0f, 0.0f, 0.0f);
            spotLight3.Theta = MathHelper.ToRadians(10.0f);
            spotLight3.Phi = MathHelper.ToRadians(20.0f);
            spotLight3.SwitchOnLight();

            UWB_XNASpotLight spotLight4 = XNAGraphicsDevice.m_TheAPI.LightManager.CreateSpotLight();
            spotLight4.Ambient = new Vector4(0.1f, 0.1f, 0.1f, 1.0f);
            spotLight4.Diffuse = new Vector4(0.0f, 0.5f, 0.0f, 1.0f);
            spotLight4.Specular = new Vector4(0.2f, 0.2f, 0.2f, 1.0f);
            spotLight4.Direction = new Vector3(0.0f, -1.0f, 0.0f);
            spotLight4.Position = new Vector3(-2.0f, 8.0f, 2.0f);
            spotLight4.Color = Color.Red.ToVector4();
            spotLight4.Attenuation = new Vector3(1.0f, 0.0f, 0.0f);
            spotLight4.Theta = MathHelper.ToRadians(10.0f);
            spotLight4.Phi = MathHelper.ToRadians(20.0f);
            spotLight4.SwitchOnLight();

            UWB_XNASpotLight spotLight5 = XNAGraphicsDevice.m_TheAPI.LightManager.CreateSpotLight();
            spotLight5.Ambient = new Vector4(0.1f, 0.1f, 0.1f, 1.0f);
            spotLight5.Diffuse = new Vector4(0.5f, 0.0f, 0.0f, 1.0f);
            spotLight5.Specular = new Vector4(0.2f, 0.2f, 0.2f, 1.0f);
            spotLight5.Direction = new Vector3(0.0f, -1.0f, 0.0f);
            spotLight5.Position = new Vector3(-2.0f, 8.0f, -2.0f);
            spotLight5.Color = Color.Red.ToVector4();
            spotLight5.Attenuation = new Vector3(1.0f, 0.0f, 0.0f);
            spotLight5.Theta = MathHelper.ToRadians(10.0f);
            spotLight5.Phi = MathHelper.ToRadians(20.0f);
            spotLight5.SwitchOnLight();
 
		}

		/// Set the scene node that is considered the "active" scene node
		/// in the model. The active node displays its bounds when drawn and
		/// can be picked with the mouse.
		/// \param node This must be a node in the m_body scene node data structure
		public void SetActiveSceneNode(UWB_SceneNode node)
		{
			m_pActiveNode = node;
			m_bActiveNodeIsPicked = false;
		}

		/// Update the state of the model. This is used to Update the objects contained in
		/// the model after a certain amount of time has passed. 
		public void UpdateSimulation()
		{
			float elapsed_seconds = m_stop_watch.getSecondsElapsed();

		}

		///
		/// Draw all of the geometry that is part of this model.
		public void DrawModel()
		{
			eLevelofDetail lod = m_DrawHelper.getLod();

			m_DrawHelper.initializeModelTransform();
	
			m_DrawHelper.pushModelTransform();
			m_RootNode.Draw(lod, m_DrawHelper);
			m_DrawHelper.popModelTransform();

		}

		///
		/// Get the bounds that this model defines
		public UWB_BoundingBox GetWorldBounds()
		{
			return m_WorldBounds;
		}

		public Rectangle WorldBounds
		{
			get { return m_WorldBounds.Rectangle; }
		}

		///
		/// Get the root node of the complete scene
		public UWB_SceneNode GetRootNode() { return m_RootNode; }

	}
}