using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using UWB_GUI;
using UWB_XNALib;

namespace XNA_UI_Controls
{
    public class PrimitiveControl : UWB_UI_Container
    {
        public const int width = 300;
        public const int height = 240;

        private UWB_UI_SliderBar m_RedSliderBar, m_GreenSliderBar, m_BlueSliderBar, m_AlphaSliderBar;
        private UWB_UI_SliderBar m_PrimitiveSlider;
        private List<UWB_UI_CheckBox> m_CheckBoxes;
        private List<UWB_UI_SliderBar> m_SliderBars;

        private UWB_UI_TreeView m_Root;
        private int m_PrevPrimitiveIndex;
        private UWB_SceneNode m_PrevNode;

        public PrimitiveControl(Point upperRight, UWB_UI_TreeView root, UWB_UI_GUI gui)
            : this(upperRight, root, gui.GetBaseContainer())
        { }

        internal PrimitiveControl(Point upperRight, UWB_UI_TreeView root, UWB_UI_Container container)
            : base(upperRight.X, upperRight.Y, width, height, Color.Azure)
        {
            m_Root = root;
            m_PrevNode = CurrentNode;
            m_SliderBars = new List<UWB_UI_SliderBar>();
            m_CheckBoxes = new List<UWB_UI_CheckBox>();

            BuildControl();
            InitSliders();
            UpdateCheckBoxes();

            AddListener(new UI_ElementListener());

            container.AddItem(new UWB_UI_Item("Primitive Control", this));
        }

        private void BuildControl()
        {
            AddItem(new UWB_UI_Item("Primitive Control Label", new UWB_UI_Label("Primitive Control", new Vector2(130, 5), this, UWB_UI_GUI.LargeFont)));

            //create the color sliderbar
            m_RedSliderBar = new UWB_UI_SliderBar(new Vector2(75, 30), 190, this);
            AddItem(new UWB_UI_Item("redLabel", new UWB_UI_Label("Red", new Vector2(10, 30), this, UWB_UI_GUI.SmallFont)));
            AddItem("redSliderBar", m_RedSliderBar);
            UWB_UI_Label redSliderBarLabel = new UWB_UI_Label(m_RedSliderBar.Value.ToString(), new Vector2(275, 30), this, UWB_UI_GUI.SmallFont);
            AddItem("redSliderBarLabel", redSliderBarLabel);
            m_RedSliderBar.AddListener(new PrimitiveControlSliderBarLabelListener(redSliderBarLabel));
            PrimitiveControlListener redSliderBarListener = new PrimitiveControlListener("Red", this);
            m_RedSliderBar.AddListener(redSliderBarListener);
            m_SliderBars.Add(m_RedSliderBar);

            m_GreenSliderBar = new UWB_UI_SliderBar(new Vector2(75, 55), 190, this);
            AddItem(new UWB_UI_Item("greenLabel", new UWB_UI_Label("Green", new Vector2(10, 55), this, UWB_UI_GUI.SmallFont)));
            AddItem("greenSliderBar", m_GreenSliderBar);
            UWB_UI_Label greenSliderBarLabel = new UWB_UI_Label(m_GreenSliderBar.Value.ToString(), new Vector2(275, 55), this, UWB_UI_GUI.SmallFont);
            AddItem("greenSliderBarLabel", greenSliderBarLabel);
            m_GreenSliderBar.AddListener(new PrimitiveControlSliderBarLabelListener(greenSliderBarLabel));
            m_GreenSliderBar.AddListener(new PrimitiveControlListener("Green", this));
            m_SliderBars.Add(m_GreenSliderBar);

            m_BlueSliderBar = new UWB_UI_SliderBar(new Vector2(75, 80), 190, this);
            AddItem(new UWB_UI_Item("blueLabel", new UWB_UI_Label("Blue", new Vector2(10, 80), this, UWB_UI_GUI.SmallFont)));
            AddItem("blueSliderBar", m_BlueSliderBar);
            UWB_UI_Label blueSliderBarLabel = new UWB_UI_Label(m_BlueSliderBar.Value.ToString(), new Vector2(275, 80), this, UWB_UI_GUI.SmallFont);
            AddItem("blueSliderBarLabel", blueSliderBarLabel);
            m_BlueSliderBar.AddListener(new PrimitiveControlSliderBarLabelListener(blueSliderBarLabel));
            m_BlueSliderBar.AddListener(new PrimitiveControlListener("Blue", this));
            m_SliderBars.Add(m_BlueSliderBar);

            m_AlphaSliderBar = new UWB_UI_SliderBar(new Vector2(75, 105), 190, this);
            AddItem(new UWB_UI_Item("alphaLabel", new UWB_UI_Label("Alpha", new Vector2(10, 105), this, UWB_UI_GUI.SmallFont)));
            AddItem("alphaSliderBar", m_AlphaSliderBar);
            UWB_UI_Label alphaSliderBarLabel = new UWB_UI_Label(m_AlphaSliderBar.Value.ToString(), new Vector2(275, 105), this, UWB_UI_GUI.SmallFont);
            AddItem("alphaSliderBarLabel", alphaSliderBarLabel);
            m_AlphaSliderBar.AddListener(new PrimitiveControlSliderBarLabelListener(alphaSliderBarLabel));
            m_AlphaSliderBar.AddListener(new PrimitiveControlListener("Alpha", this));
            m_SliderBars.Add(m_AlphaSliderBar);

            m_PrimitiveSlider = new UWB_UI_SliderBar(new Vector2(75, 130), 190, this);
            AddItem(new UWB_UI_Item("selectorLabel", new UWB_UI_Label("Primitive\nSelect", new Vector2(10, 125), this, UWB_UI_GUI.SmallFont)));
            AddItem("selectorSliderBar", m_PrimitiveSlider);
            UWB_UI_Label selectorSliderBarLabel = new UWB_UI_Label(m_PrimitiveSlider.Value.ToString(), new Vector2(275, 130), this, UWB_UI_GUI.SmallFont);
            AddItem("selectorSliderBarLabel", selectorSliderBarLabel);
            m_PrimitiveSlider.AddListener(new PrimitiveControlSliderBarLabelListener(selectorSliderBarLabel));
            m_PrimitiveSlider.AddListener(new PrimitiveControlListener("Primitive", this));
            m_SliderBars.Add(m_PrimitiveSlider);

            //add checkBoxes
            UWB_UI_CheckBox chkBoxVisible = new UWB_UI_CheckBox("Visibility", new Vector2(5, 160), this, UWB_UI_GUI.SmallFont);
            AddItem("Visibility", chkBoxVisible);
            chkBoxVisible.AddListener(new XNACheckBoxListener("Visibility", this));
            m_CheckBoxes.Add(chkBoxVisible);

            UWB_UI_CheckBox chkBoxShading = new UWB_UI_CheckBox("Shading", new Vector2(5, 175), this, UWB_UI_GUI.SmallFont);
            AddItem("Shading", chkBoxShading);
            chkBoxShading.AddListener(new XNACheckBoxListener("Shading", this));
            m_CheckBoxes.Add(chkBoxShading);

            UWB_UI_CheckBox chkBoxDrawBoundingBox = new UWB_UI_CheckBox("Draw Bounding Box", new Vector2(5, 190), this, UWB_UI_GUI.SmallFont);
            AddItem("Draw Bounding Box", chkBoxDrawBoundingBox);
            chkBoxDrawBoundingBox.AddListener(new XNACheckBoxListener("Draw Bounding Box", this));
            m_CheckBoxes.Add(chkBoxDrawBoundingBox);

            UWB_UI_CheckBox chkBoxWireframe = new UWB_UI_CheckBox("Wireframe", new Vector2(5, 205), this, UWB_UI_GUI.SmallFont);
            AddItem("Wireframe", chkBoxWireframe);
            chkBoxWireframe.AddListener(new XNACheckBoxListener("Wireframe", this));
            m_CheckBoxes.Add(chkBoxWireframe);

            UWB_UI_CheckBox chkBoxBlending = new UWB_UI_CheckBox("Blending", new Vector2(150, 160), this, UWB_UI_GUI.SmallFont);
            AddItem("Blending", chkBoxBlending);
            chkBoxBlending.AddListener(new XNACheckBoxListener("Blending", this));
            m_CheckBoxes.Add(chkBoxBlending);

            UWB_UI_CheckBox chkBoxDrawBoundingSphere = new UWB_UI_CheckBox("Draw Bounding Sphere", new Vector2(150, 175), this, UWB_UI_GUI.SmallFont);
            AddItem("Draw Bounding Sphere", chkBoxDrawBoundingSphere);
            chkBoxDrawBoundingSphere.AddListener(new XNACheckBoxListener("Draw Bounding Sphere", this));
            m_CheckBoxes.Add(chkBoxDrawBoundingSphere);

            UWB_UI_CheckBox chkBoxTexture = new UWB_UI_CheckBox("Texture", new Vector2(150, 190), this, UWB_UI_GUI.SmallFont);
            AddItem("Texture", chkBoxTexture);
            chkBoxTexture.AddListener(new XNACheckBoxListener("Texture", this));
            m_CheckBoxes.Add(chkBoxTexture);

            UWB_UI_CheckBox chkBoxDefaultLighting = new UWB_UI_CheckBox("Default Lighting", new Vector2(150, 205), this, UWB_UI_GUI.SmallFont);
            AddItem("Default Lighting", chkBoxDefaultLighting);
            chkBoxDefaultLighting.AddListener(new XNACheckBoxListener("Default Lighting", this));
            m_CheckBoxes.Add(chkBoxDefaultLighting);

            UWB_UI_CheckBox chkBoxShowRotateAxis = new UWB_UI_CheckBox("Show Rotate Axis", new Vector2(150, 220), this, UWB_UI_GUI.SmallFont);
            AddItem("Show Rotate Axis", chkBoxShowRotateAxis);
            chkBoxShowRotateAxis.AddListener(new XNACheckBoxListener("Show Rotate Axis", this));
            m_CheckBoxes.Add(chkBoxShowRotateAxis);
        }

        private void InitSliders()
        {
            foreach (UWB_UI_SliderBar slider in m_SliderBars)
            {
                slider.SetMinValue(0);
                slider.SetMaxValue(255);
            }

            UpdatePrimitiveSlider();
            UpdateColorSliders();
        }

        private void UpdatePrimitiveSlider()
        {
            m_PrimitiveSlider.SetValue(0);
            m_PrimitiveSlider.SetMaxValue(PrimitiveCount - 1);
            m_PrevPrimitiveIndex = m_PrimitiveSlider.Value;
        }

        private void UpdateColorSliders()
        {
            if (CurrentPrimitive != null)
            {
                Color color = CurrentPrimitive.getFlatColor();
                m_RedSliderBar.SetValue((int)(color.R));
                m_GreenSliderBar.SetValue((int)(color.G));
                m_BlueSliderBar.SetValue((int)(color.B));
                m_AlphaSliderBar.SetValue((int)(color.A));
            }
        }

        public override void Update(GameTime gameTime)
        {
            foreach (UWB_UI_SliderBar slider in m_SliderBars)
            {
                slider.Update(gameTime);
            }
        }

        /// <summary>
        /// Called when the selected node changes (in the SceneTreeControl)
        /// Resets primitive slider and updates control values
        /// </summary>
        internal void NodeChangeUpdate()
        {
            UpdatePrimitiveSlider();
            PrimitiveChangeUpdate();
        }

        /// <summary>
        /// Called when the primitive slider value is changed
        /// Updates control values to reflect new primitive
        /// </summary>
        internal void PrimitiveChangeUpdate()
        {
            UpdateColorSliders();
            UpdateCheckBoxes();
        }

        private void UpdateCheckBoxes()
        {
            if (CurrentPrimitive != null)
            {
                foreach (UWB_UI_CheckBox chkBox in m_CheckBoxes)
                {
                    switch (chkBox.Text)
                    {
                        case "Visibility":
                            chkBox.IsSelected = CurrentPrimitive.isVisible();
                            break;
                        case "Shading":
                            chkBox.IsSelected = (CurrentPrimitive.getShadeMode() == eShadeMode.smGouraud);
                            break;
                        case "Draw Bounding Box":
                            chkBox.IsSelected = CurrentPrimitive.isDrawingBoundingVolume();
                            break;
                        case "Wireframe":
                            chkBox.IsSelected = (CurrentPrimitive.getFillMode() == eFillMode.fmWireframe);
                            break;
                        case "Show Pivot":
                            chkBox.IsSelected = CurrentNode.isPivotVisible();
                            break;
                        // the following are not yet implemnented
                        case "Blending":
                        case "Draw Bounding Sphere":
                        case "Texture":
                        case "Default Lighting":
                        case "Show Rotate Axis":
                        default:
                            chkBox.IsSelected = false;
                            break;
                    }
                }
            }
        }

        /// <summary>
        /// Used by UpdatePrimitiveSlider() to set maximum value of the
        /// primitive slider bar
        /// </summary>
        private int PrimitiveCount
        {
            get
            {
                UWB_Primitive primitive = CurrentNode.getPrimitive();

                
                if (primitive != null && primitive.GetType() == typeof(UWB_PrimitiveList))
                    return ((UWB_PrimitiveList)primitive).count();

                return 1;
            }
        }

        internal UWB_Primitive CurrentPrimitive
        {
            get
            {
                UWB_Primitive primitive = CurrentNode.getPrimitive();

                if (primitive != null)
                {
                    if (primitive.GetType() == typeof(UWB_PrimitiveList))
                        primitive = ((UWB_PrimitiveList)primitive).primitiveAt(m_PrimitiveSlider.Value);
                }

                return primitive;
            }
        }

        private UWB_SceneNode CurrentNode
        {
            get { return (UWB_SceneNode)m_Root.GetCurrentSelectedNode().Value; }
        }
    }
}
