using System;
using System.Collections.Generic;
using System.Text;
using UWB_GUI;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using UWB_XNALib;

namespace XNA_UI_Controls
{
    public class PrimitiveControlListener : UWB_UI_Listener
    {
        private string m_SliderBarType;
        private PrimitiveControl m_Control;

        public PrimitiveControlListener(string type, PrimitiveControl primitiveControl)
        {
            m_SliderBarType = type;
            m_Control = primitiveControl;
        }

        public override bool ValueChanged(UWB_UI_Element obj)
        {
            // slider value changed or primitive slider value changed
            if (obj.GetType() == typeof(UWB_UI_SliderBar) && m_Control.CurrentPrimitive != null)
            {
                // retrieve new color and set primitive (unless it was the primitive slider that changed)
                Color selectedObjectColor = m_Control.CurrentPrimitive.getFlatColor();
                int val = ((UWB_UI_SliderBar)obj).Value;
                switch (m_SliderBarType)
                {
                    case "Red":
                        selectedObjectColor = new Color((byte)val, selectedObjectColor.G, selectedObjectColor.B, selectedObjectColor.A);
                        break;
                    case "Green":
                        selectedObjectColor = new Color(selectedObjectColor.R, (byte)val, selectedObjectColor.B, selectedObjectColor.A);
                        break;
                    case "Blue":
                        selectedObjectColor = new Color(selectedObjectColor.R, selectedObjectColor.G, (byte)val, selectedObjectColor.A);
                        break;
                    case "Alpha":
                        selectedObjectColor = new Color(selectedObjectColor.R, selectedObjectColor.G, selectedObjectColor.B, (byte)val);
                        break;
                    case "Primitive":
                        // primitive changed
                        UpdateControlValues();
                        break;
                    default:
                        return false;
                }
                m_Control.CurrentPrimitive.setFlatColor(selectedObjectColor);
            }
            // primitive changed
            if(obj.GetType() == typeof(UWB_UI_TreeView))
                UpdateControlValues();

            return false;
        }

        private void UpdateControlValues()
        {
            m_Control.PrimitiveChangeUpdate();
        }
    }
}
