using System;
using System.Collections.Generic;
using System.Text;
using UWB_GUI;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using UWB_XNALib;

namespace XNA_UI_Controls
{
    /// <summary>
    /// Builds a Scene Tree from a UWB_UI_TreeView with data from a
    /// UWB_SceneNode
    /// </summary>
    public class SceneTreeControl : UWB_UI_Container
    {
        public const int width = 200;
        public const int height = 300;

        private UWB_UI_TreeView m_ObjectsTreeView;

        private XformInfoControl m_XformInfoControl;
        private PrimitiveControl m_PrimitiveControl;

        internal SceneTreeControl(Point upperRight, UWB_SceneNode root, UWB_UI_GUI gui)
            : this(upperRight, root, gui.GetBaseContainer())
        { }

        internal SceneTreeControl(Point upperRight, UWB_SceneNode root, UWB_UI_Container container)
            : base(upperRight.X, upperRight.Y, width, height, Color.Azure)
        {
            m_XformInfoControl = null;
            m_PrimitiveControl = null;
            m_ObjectsTreeView = new UWB_UI_TreeView(root.Name, root, this, UWB_UI_GUI.SmallFont);

            BuildControl(root);

            m_ObjectsTreeView.AddListener(new SceneTreeListener(this));
            AddListener(new UI_ElementListener());
            container.AddItem(new UWB_UI_Item("SceneTreeControl", this));
        }

        private void BuildControl(UWB_SceneNode root)
        {
            m_ObjectsTreeView.Position = new Vector2(5, 30);

            AddItem(new UWB_UI_Item("Objects Label", new UWB_UI_Label("Objects", new Vector2(5, 5), this, UWB_UI_GUI.LargeFont)));

            UWB_UI_TreeViewNode treeNode = null;

            for (int i = 0; i < root.numChildren(); i++)
            {
                BuildTree(root.getChildNode(i), ref treeNode);
                if (treeNode != null)
                {
                    m_ObjectsTreeView.AddChild(treeNode);
                    treeNode = null;
                }
            }
            m_ObjectsTreeView.SetSelectedNode(root.Name);

            AddItem(new UWB_UI_Item("ObjectsTreeView", m_ObjectsTreeView));
        }

        private void BuildTree(UWB_SceneNode sceneNode, ref UWB_UI_TreeViewNode treeNode)
        {
            if (treeNode == null)
                treeNode = new UWB_UI_TreeViewNode(sceneNode.Name, sceneNode, UWB_UI_GUI.SmallFont, this);
            else if (sceneNode.numChildren() == 0)
                treeNode.AddChild(sceneNode.Name, sceneNode, UWB_UI_GUI.SmallFont);

            UWB_UI_TreeViewNode treeBranch = null;
            for (int i = 0; i < sceneNode.numChildren(); i++)
            {
                BuildTree(sceneNode.getChildNode(i), ref treeBranch);
                if (treeBranch != null)
                {
                    treeNode.AddChild(treeBranch);
                    treeBranch = null;
                }
            }
        }

        internal UWB_UI_TreeView SceneTree
        {
            get { return m_ObjectsTreeView; }
        }

        internal XformInfoControl XformInfoControl
        {
            set { m_XformInfoControl = value; }
        }

        internal PrimitiveControl PrimitiveControl
        {
            set { m_PrimitiveControl = value; }
        }

        internal void UpdateControls()
        {
            if(m_XformInfoControl != null)
                m_XformInfoControl.UpdateControlValues();
            if(m_PrimitiveControl != null)
                m_PrimitiveControl.NodeChangeUpdate();
        }
    }
}
