using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using UWB_GUI;
using Microsoft.Xna.Framework.Graphics;
using UWB_XNALib;

namespace XNA_UI_Controls
{
    public class SceneTreeXformInfoPrimitiveControl : UWB_UI_Container
    {
        public const int width = XformInfoControl.width;
        public const int height = SceneTreeControl.height + XformInfoControl.height + PrimitiveControl.height;

        private SceneTreeControl m_SceneTree;
        private XformInfoControl m_XformInfoControl;
        private PrimitiveControl m_PrimitiveControl;

        public SceneTreeXformInfoPrimitiveControl(Point upperRight, UWB_SceneNode rootNode, UWB_UI_GUI m_GUI)
            : this(upperRight, rootNode, m_GUI.GetBaseContainer())
        { }

        public SceneTreeXformInfoPrimitiveControl(Point upperRight, UWB_SceneNode rootNode, UWB_UI_Container container)
            : base(upperRight.X, upperRight.Y, width, height, Color.TransparentWhite)
        {
            ClearBackground = false;

            Point treeUpperRight = new Point(upperRight.X + XformInfoControl.width - SceneTreeControl.width, upperRight.Y);
            Point xformUpperRight = new Point(upperRight.X, upperRight.Y + SceneTreeControl.height);
            Point primitiveUpperRight = new Point(xformUpperRight.X, xformUpperRight.Y + XformInfoControl.height);
            
            m_SceneTree = new SceneTreeControl(treeUpperRight, rootNode, this);
            m_XformInfoControl = new XformInfoControl(xformUpperRight, m_SceneTree.SceneTree, this);
            m_PrimitiveControl = new PrimitiveControl(primitiveUpperRight, m_SceneTree.SceneTree, this);

            m_SceneTree.XformInfoControl = m_XformInfoControl;
            m_SceneTree.PrimitiveControl = m_PrimitiveControl;

            BackgroundColor = new Color(new Vector4(Color.Azure.ToVector3(), 0.75f));

            container.AddItem("SceneTreeXformPrimitive Control", this);
        }

        protected override bool ExecuteMousePressedListeners()
        {
            return (m_PrimitiveControl.SetMousePressed()
                || m_SceneTree.SetMousePressed()
                || m_XformInfoControl.SetMousePressed());
        }

        protected override bool ExecuteMouseDraggedListeners()
        {
            return (m_PrimitiveControl.SetMouseDragged()
                || m_SceneTree.SetMouseDragged()
                || m_XformInfoControl.SetMouseDragged());
        }

        protected override bool ExecuteMouseReleasedListeners()
        {
            return (m_PrimitiveControl.SetMouseReleased()
                || m_SceneTree.SetMouseReleased()
                || m_XformInfoControl.SetMouseReleased());
        }

        protected override bool ExecuteMouseMovedListeners()
        {
            return (m_PrimitiveControl.SetMouseMoved()
                || m_SceneTree.SetMouseMoved()
                || m_XformInfoControl.SetMouseMoved());
        }

        public override Color BackgroundColor
        {
            get { return m_PrimitiveControl.BackgroundColor; }
            set
            {
                m_PrimitiveControl.BackgroundColor = value;
                m_SceneTree.BackgroundColor = value;
                m_XformInfoControl.BackgroundColor = value;
            }
        }

        public Vector2 TranslateRange
        {
            get { return m_XformInfoControl.TranslateRange; }
            set { m_XformInfoControl.TranslateRange = value; }
        }

        public Vector2 ScaleRange
        {
            get { return m_XformInfoControl.ScaleRange; }
            set { m_XformInfoControl.ScaleRange = value; }
        }

        public Vector2 RotateRange
        {
            get { return m_XformInfoControl.RotateRange; }
            set { m_XformInfoControl.RotateRange = value; }
        }

        public Vector2 PivotRange
        {
            get { return m_XformInfoControl.PivotRange; }
            set { m_XformInfoControl.PivotRange = value; }
        }
    }
}
