using System;
using System.Collections.Generic;
using System.Text;
using UWB_GUI;
using UWB_XNALib;

namespace XNA_UI_Controls
{
    public class XNACheckBoxListener : UWB_UI_Listener
    {
        private string m_Type;
        private XformInfoControl m_XformInfoControl;
        private PrimitiveControl m_PrimitiveControl;

        public XNACheckBoxListener(string type, XformInfoControl xFormInfoControl)
            : this(type, xFormInfoControl, null)
        { }

        public XNACheckBoxListener(string type, PrimitiveControl primitiveControl)
            : this(type, null, primitiveControl)
        { }

        private XNACheckBoxListener(string type, XformInfoControl xFormInfoControl, PrimitiveControl primitiveControl)
        {
            m_Type = type;
            m_XformInfoControl = xFormInfoControl;
            m_PrimitiveControl = primitiveControl;
        }

        public override bool CheckedChange(UWB_UI_Element obj)
        {
            bool selected = ((UWB_UI_CheckBox)obj).IsSelected;

            switch (m_Type)
            {
                case "Visibility":
                    CurrentPrimitive.setVisible(selected);
                    break;
                case "Shading":
                    CurrentPrimitive.setShadeMode(selected ? eShadeMode.smGouraud : eShadeMode.smFlat);
                    break;
                case "Blending":
                    break;
                case "Draw Bounding Box":
                    CurrentPrimitive.setDrawBoundingVolume(selected);
                    break;
                case "Draw Bounding Sphere":
                    break;
                case "Wireframe":
                    CurrentPrimitive.setFillMode(selected ? eFillMode.fmWireframe : eFillMode.fmSolid);
                    break;
                case "Texture":
                    break;
                case "Default Lighting":
                    break;
                case "Show Rotate Axis":
                    break;
                case "Show Pivot":
                    if(CurrentNode != null)
                        CurrentNode.setPivotVisible(selected);
                    break;
            }
            return false;
        }

        public override bool ValueChanged(UWB_UI_Element obj)
        {
            switch (m_Type)
            {
                case "Visibility":
                    ((UWB_UI_CheckBox)obj).IsSelected = CurrentPrimitive.isVisible();
                    break;
                case "Shading":
                    ((UWB_UI_CheckBox)obj).IsSelected = (CurrentPrimitive.getShadeMode() == eShadeMode.smGouraud);
                    break;
                case "Blending":
                    break;
                case "Draw Bounding Box":
                    ((UWB_UI_CheckBox)obj).IsSelected = CurrentPrimitive.isDrawingBoundingVolume();
                    break;
                case "Draw Bounding Sphere":
                    break;
                case "Wireframe":
                    ((UWB_UI_CheckBox)obj).IsSelected = (CurrentPrimitive.getFillMode() == eFillMode.fmWireframe);
                    break;
                case "Texture":
                    break;
                case "Default Lighting":
                    break;
                case "Show Rotate Axis":
                    break;
                case "Show Pivot":
                    if (CurrentNode != null)
                        ((UWB_UI_CheckBox)obj).IsSelected = CurrentNode.isPivotVisible();
                    break;
            }
            return false;
        }

        private UWB_SceneNode CurrentNode
        {
            get { return (m_XformInfoControl != null) ? m_XformInfoControl.CurrentNode : null; }
        }

        private UWB_Primitive CurrentPrimitive
        {
            get
            {
                if (m_PrimitiveControl != null)
                    return m_PrimitiveControl.CurrentPrimitive;
                return null;
            }
        }
    }

}
