using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework;
using UWB_GUI;
using UWB_XNALib;

namespace XNA_UI_Controls
{
    public class XformInfoControl : UWB_UI_Container
    {
        internal struct sNodeInfo
        {
            public string name;
            public int index;
            public bool pivotVisible;
            public UWB_XFormInfo xFormInfo;
        }

        public const int width = 300;
        public const int height = 145;
        public const int margin = 5;
        private const string initRadioButton = "Translate";

        private Vector2 m_TranslateRange = new Vector2(-50f, 50f);
        private Vector2 m_ScaleRange = new Vector2(-10f, 10f);
        private Vector2 m_RotateRange = new Vector2(-360f, 360f);
        private Vector2 m_PivotRange = new Vector2(-50f, 50f);

        private Vector3 m_CurrentRotation = new Vector3(0.0f);

        private UWB_UI_RadioButtonSet m_controlsRadioSet;
        private UWB_UI_SliderBar m_xSliderBar;
        private UWB_UI_SliderBar m_ySliderBar;
        private UWB_UI_SliderBar m_zSliderBar;
        private UWB_UI_CheckBox m_PivotCheckBox;
        private UWB_UI_Button m_ResetButton;

        private List<sNodeInfo> m_ResetInfo;
        private UWB_UI_TreeView m_Root;
        private List<UWB_XFormInfo> m_Xforms;
        private int m_Index = 0;

        public XformInfoControl(Point upperRight, UWB_XFormInfo xform, UWB_UI_GUI gui)
            : this(upperRight, xform, gui.GetBaseContainer())
        { }
        public XformInfoControl(Point upperRight, UWB_UI_TreeView root, UWB_UI_GUI gui)
            : this(upperRight, root, gui.GetBaseContainer())
        { }
        public XformInfoControl(Point upperRight, List<UWB_XFormInfo> xformList, UWB_UI_GUI gui)
            : this(upperRight, xformList, gui.GetBaseContainer())
        { }

        internal XformInfoControl(Point upperRight, UWB_XFormInfo xform, UWB_UI_Container container)
            : this(upperRight, container)
        {
            m_Xforms = new List<UWB_XFormInfo>();
            m_Xforms.Add(xform);
            SaveResetInfo();

            UpdateControlValues();
        }

        internal XformInfoControl(Point upperRight, UWB_UI_TreeView root, UWB_UI_Container container)
            : this(upperRight, container)
        {
            m_Root = root;
            SaveResetInfo((UWB_SceneNode)m_Root.GetRoot().Value);

            UpdateControlValues();
        }

        internal XformInfoControl(Point upperRight, List<UWB_XFormInfo> xformList, UWB_UI_Container container)
            : this(upperRight, container)
        {
            m_Xforms = xformList;
            SaveResetInfo();

            UpdateControlValues();
        }

        private XformInfoControl(Point upperRight, UWB_UI_Container container)
            : base(upperRight.X, upperRight.Y, width, height, Color.Azure)
        {
            m_Root = null;
            m_Xforms = null;
            m_ResetInfo = new List<sNodeInfo>();

            BuildXformInfoControl();

            AddListener(new UI_ElementListener());
            container.AddItem(new UWB_UI_Item("XformInfoControl", this));
        }

        private void BuildXformInfoControl()
        {
            AddItem(new UWB_UI_Item("XformInfo Label", new UWB_UI_Label("XformInfo Control", new Vector2(width - 180 - margin, margin), this, UWB_UI_GUI.LargeFont)));

            // radio set
            AddItem(new UWB_UI_Item("Controls Label", new UWB_UI_Label("Controls", new Vector2(20, 28), this, UWB_UI_GUI.SmallFont)));

            m_controlsRadioSet = new UWB_UI_RadioButtonSet();
            m_controlsRadioSet.AddRadioButton(new UWB_UI_RadioButton("Translate", new Vector2(10, 45), this, UWB_UI_GUI.SmallFont));
            m_controlsRadioSet.AddRadioButton(new UWB_UI_RadioButton("Scale", new Vector2(10, 60), this, UWB_UI_GUI.SmallFont));
            m_controlsRadioSet.AddRadioButton(new UWB_UI_RadioButton("Rotate", new Vector2(10, 75), this, UWB_UI_GUI.SmallFont));
            m_controlsRadioSet.AddRadioButton(new UWB_UI_RadioButton("Pivot", new Vector2(10, 90), this, UWB_UI_GUI.SmallFont));
            m_controlsRadioSet.SetSelectedRadioButton(initRadioButton);
            m_controlsRadioSet.AddListener(new XformInfoControlListener("", this));
            AddItem(new UWB_UI_Item("ControlsRadioSet", m_controlsRadioSet));

            //create the x,y,z sliderbars
            AddItem(new UWB_UI_Item("Transform Label", new UWB_UI_Label("Transform", new Vector2(160, 28), this, UWB_UI_GUI.SmallFont)));
            m_xSliderBar = new UWB_UI_SliderBar(new Vector2(105, 45), 160, this);
            m_xSliderBar.SetValue(0);
            AddItem("xSliderBar", m_xSliderBar);
            AddItem(new UWB_UI_Item("XLabel", new UWB_UI_Label("X/R", new Vector2(85, 45), this, UWB_UI_GUI.SmallFont)));
            UWB_UI_Label xSliderBarLabel = new UWB_UI_Label((m_xSliderBar.Value / 10f).ToString("0.0"), new Vector2(width - 25, 45), this, UWB_UI_GUI.SmallFont);
            AddItem("xSliderBarLabel", xSliderBarLabel);
            m_xSliderBar.AddListener(new TransformSliderBarLabelListener(xSliderBarLabel));
            m_xSliderBar.AddListener(new XformInfoControlListener("X", this));

            m_ySliderBar = new UWB_UI_SliderBar(new Vector2(105, 70), 160, this);
            m_ySliderBar.SetValue(0);
            AddItem("ySliderBar", m_ySliderBar);
            AddItem(new UWB_UI_Item("YLabel", new UWB_UI_Label("Y", new Vector2(90, 70), this, UWB_UI_GUI.SmallFont)));
            UWB_UI_Label ySliderBarLabel = new UWB_UI_Label((m_ySliderBar.Value / 10f).ToString("0.0"), new Vector2(width - 25, 70), this, UWB_UI_GUI.SmallFont);
            AddItem("ySliderBarLabel", ySliderBarLabel);
            m_ySliderBar.AddListener(new TransformSliderBarLabelListener(ySliderBarLabel));
            m_ySliderBar.AddListener(new XformInfoControlListener("Y", this));

            m_zSliderBar = new UWB_UI_SliderBar(new Vector2(105, 95), 160, this);
            m_zSliderBar.SetValue(0);
            AddItem("zSliderBar", m_zSliderBar);
            AddItem(new UWB_UI_Item("ZLabel", new UWB_UI_Label("Z", new Vector2(90, 95), this, UWB_UI_GUI.SmallFont)));
            UWB_UI_Label zSliderBarLabel = new UWB_UI_Label((m_zSliderBar.Value / 10f).ToString("0.0"), new Vector2(width - 25, 95), this, UWB_UI_GUI.SmallFont);
            AddItem("zSliderBarLabel", zSliderBarLabel);
            m_zSliderBar.AddListener(new TransformSliderBarLabelListener(zSliderBarLabel));
            m_zSliderBar.AddListener(new XformInfoControlListener("Z", this));

            m_PivotCheckBox = new UWB_UI_CheckBox("Show Pivot", new Vector2(10, 120), this, UWB_UI_GUI.SmallFont);
            m_PivotCheckBox.AddListener(new XNACheckBoxListener("Show Pivot", this));
            AddItem(new UWB_UI_Item("Show Pivot", m_PivotCheckBox));

            UpdateSliderRanges();

            m_ResetButton = new UWB_UI_Button("Reset", new Vector2(222, 145 - 27), this, UWB_UI_GUI.SmallFont);
            m_ResetButton.AddListener(new ResetButtonListener(this));
            AddItem(new UWB_UI_Item("Reset Button", m_ResetButton));
        }

        internal void UpdateControlValues()
        {
            SetSliderValues(CurrentXFormValues);
            m_PivotCheckBox.IsSelected = CurrentNode.isPivotVisible();
        }

        private void SetSliderValues(Vector3 values)
        {
            m_xSliderBar.SetValue((int)(values.X * 10));
            m_ySliderBar.SetValue((int)(values.Y * 10));
            m_zSliderBar.SetValue((int)(values.Z * 10));
        }

        private void SetSliderRange(int min, int max)
        {
            m_xSliderBar.SetMinValue(min);
            m_ySliderBar.SetMinValue(min);
            m_zSliderBar.SetMinValue(min);
            m_xSliderBar.SetMaxValue(max);
            m_ySliderBar.SetMaxValue(max);
            m_zSliderBar.SetMaxValue(max);
        }

        internal void UpdateSliderRanges()
        {
            switch (CurrentRadioButton)
            {
                case "Translate":
                    SetSliderRange((int)(TranslateRange.X * 10f), (int)(TranslateRange.Y * 10f));
                    break;
                case "Scale":
                    SetSliderRange((int)(ScaleRange.X * 10f), (int)(ScaleRange.Y * 10f));
                    break;
                case "Rotate":
                    SetSliderRange((int)(RotateRange.X * 10f), (int)(RotateRange.Y * 10f));
                    break;
                case "Pivot":
                    SetSliderRange((int)(PivotRange.X * 10f), (int)(PivotRange.Y * 10f));
                    break;
                default:
                    break;
            }
        }

        private void SaveResetInfo(UWB_SceneNode curNode)
        {
            sNodeInfo curInfo = new sNodeInfo();

            curInfo.name = curNode.Name;
            curInfo.pivotVisible = curNode.isPivotVisible();
            curInfo.xFormInfo = CopyXFormInfo(curNode.getXFormInfo());
            m_ResetInfo.Add(curInfo);

            for (int i = 0; i < curNode.numChildren(); i++)
                SaveResetInfo(curNode.getChildNode(i));
        }

        private void SaveResetInfo()
        {
            if (m_Xforms != null)
            {
                for (int i = 0; i < m_Xforms.Count; i++)
                {
                    sNodeInfo curNodeInfo = new sNodeInfo();
                    curNodeInfo.index = i;
                    curNodeInfo.xFormInfo = CopyXFormInfo(m_Xforms[i]);
                    m_ResetInfo.Add(curNodeInfo);
                }
            }
        }

        private UWB_XFormInfo CopyXFormInfo(UWB_XFormInfo source)
        {
            UWB_XFormInfo xform = new UWB_XFormInfo();
            xform.SetTranslation(source.GetTranslation());
            xform.SetScale(source.GetScale());
            Quaternion q = source.GetRotationQuat();
            xform.SetRotationQuat(q);
            xform.SetPivot(source.GetPivot());
            return xform;
        }

        internal void ResetCurrentXform()
        {
            UWB_SceneNode curNode = CurrentNode;
            if (curNode != null)
            {
                int i = 0;
                bool found = false;

                while (i < m_ResetInfo.Count && !found)
                {
                    if (m_ResetInfo[i].name == curNode.Name)
                    {
                        curNode.setXFormInfo(CopyXFormInfo(m_ResetInfo[i].xFormInfo));
                        curNode.setPivotVisible(m_ResetInfo[i].pivotVisible);
                        UpdateControlValues();
                        found = true;
                    }
                    i++;
                }
            }
            else if (m_Xforms != null)
            {
                int i = 0;
                bool found = false;

                while (i < m_Xforms.Count && !found)
                {
                    if (m_ResetInfo[i].index == Index)
                    {
                        curNode.setXFormInfo(CopyXFormInfo(m_ResetInfo[i].xFormInfo));
                        UpdateControlValues();
                        found = true;
                    }
                    i++;
                }
            }
        }

        public override bool SetMousePressed()
        {
            Point mousePosition = new Point((int)UWB_UI_Mouse.Position.X, (int)UWB_UI_Mouse.Position.Y);
            return base.LeftMouseButtonDown(mousePosition);
        }

        public override bool SetMouseReleased()
        {
            Point mousePosition = new Point((int)UWB_UI_Mouse.Position.X, (int)UWB_UI_Mouse.Position.Y);
            return base.LeftMouseButtonReleased(mousePosition);
        }

        public override bool SetMouseMoved()
        {
            Point mousePosition = new Point((int)UWB_UI_Mouse.Position.X, (int)UWB_UI_Mouse.Position.Y);
            return base.MouseMoved(mousePosition);
        }

        /// <summary>
        /// Increments the index used to access the List of UWB_XformInfo objects.
        /// (Only useful when the control is constructed with such a list)
        /// </summary>
        protected override bool MiddleMouseButtonDown(Point mousePosition)
        {
            if (m_Bounds.Contains(mousePosition))
            {
                Index++;
                ExecuteUpdateListeners();
            }
            return false;
        }

        private int Index
        {
            get { return m_Index; }
            set
            {
                if (m_Xforms != null)
                    m_Index = (value >= m_Xforms.Count) ? 0 : value;
            }
        }

        internal string CurrentRadioButton
        {
            get { return m_controlsRadioSet.GetSelectedRadioButton().Text; }
        }

        internal UWB_SceneNode CurrentNode
        {
            get
            {
                return (m_Root == null) ? null : (UWB_SceneNode)m_Root.GetCurrentSelectedNodeValue();
            }
        }

        internal UWB_XFormInfo CurrentXFormInfo
        {
            get
            {
                if (m_Xforms != null)
                    return m_Xforms[Index];
                else if (m_Root != null)
                    return CurrentNode.getXFormInfo();

                return null;
            }
        }

        internal Vector3 CurrentXFormValues
        {
            get
            {
                if (CurrentXFormInfo != null)
                {
                    switch (CurrentRadioButton)
                    {
                        case "Translate":
                            return CurrentXFormInfo.GetTranslation();
                        case "Rotate":
                            return m_CurrentRotation;
                        case "Scale":
                            return CurrentXFormInfo.GetScale();
                        case "Pivot":
                            return CurrentXFormInfo.GetPivot();
                        default:
                            break;
                    }
                }
                return Vector3.Zero;
            }

            set
            {
                if (CurrentXFormInfo != null)
                {
                    switch (CurrentRadioButton)
                    {
                        case "Translate":
                            CurrentXFormInfo.SetTranslation(value);
                            break;
                        case "Rotate":
                            Vector3 dR = m_CurrentRotation - value;
                            CurrentXFormInfo.UpdateRotationXByDegree(dR.X);
                            CurrentXFormInfo.UpdateRotationYByDegree(dR.Y);
                            CurrentXFormInfo.UpdateRotationZByDegree(dR.Z);
                            m_CurrentRotation = value;
                            break;
                        case "Scale":
                            CurrentXFormInfo.SetScale(value);
                            break;
                        case "Pivot":
                            CurrentXFormInfo.SetPivot(value);
                            break;
                        default:
                            break;
                    }
                }
            }
        }

        public Vector2 TranslateRange
        {
            get { return m_TranslateRange; }
            set
            {
                m_TranslateRange = value;
                UpdateSliderRanges();
            }
        }

        public Vector2 ScaleRange
        {
            get { return m_ScaleRange; }
            set
            {
                m_ScaleRange = value;
                UpdateSliderRanges();
            }
        }

        public Vector2 RotateRange
        {
            get { return m_RotateRange; }
            set
            {
                m_RotateRange = value;
                UpdateSliderRanges();
            }
        }

        public Vector2 PivotRange
        {
            get { return m_PivotRange; }
            set
            {
                m_PivotRange = value;
                UpdateSliderRanges();
            }
        }
    }
}
